define([
        "dojo/_base/declare",
        "dojo/_base/array",
        "dojo/_base/lang",
        "dojo/Deferred",
        "dojo/store/util/QueryResults",
        "aps/SimpleQueryEngine",
        "aps/Store",
        "./rql/Memory.js",
        "./rql/js-array.js",
        "./CustomizableStore.js",
        "./utils.js"
], function (
        declare,
        arr,
        lang,
        Deferred,
        queryResults,
        SimpleQueryEngine,
        Store,
        Memory,
        jsArray,
        CustomizableStore,
        utils
) {

return declare([CustomizableStore], {
    _memory: new Memory({data: []}),

    customColumns: null,

        //queryEngine: SimpleQueryEngine,
        queryEngine: jsArray.query,

    constructor: function() {
        if (this.customColumns != null) {
            for (var key in this.customColumns) {
                if(this.customColumns.hasOwnProperty(key)){
                    (function (columnKey, column) {
                        if ("map" in column) {
                            if (!("get" in column)) {
                                var map = column.map;
                                var targetAttr = "targetAttr" in column ? column.targetAttr : null;
                                if (map instanceof Store) {
                                    column["get"] = targetAttr == null ? function (item) {
                                            return  map.get(item[column.sourceAttr]);
                                        } : function (item) {
                                            return  map.get(item[column.sourceAttr])[targetAttr];
                                        };
                                } else {
                                    column["get"] = targetAttr == null ? function (item) {
                                            return column.sourceAttr in item ? map[item[column.sourceAttr]] : undefined;
                                        } : function (item) {
                                            return (column.sourceAttr in item) && (item[column.sourceAttr] in map) ? map[item[column.sourceAttr]][targetAttr] : undefined;
                                        };
                                }
                            }
                            if (!("sort" in column) && "additional_sort" in column) {
                                column["sort"] = function (a, b) {
                                    if (a[columnKey] == b[columnKey]) {
                                        if (a[column.sourceAttr] == b[column.sourceAttr])
                                            return column["additional_sort"](a, b);
                                        return utils.compareStr(a[column.sourceAttr], b[column.sourceAttr]);
                                    }
                                    return utils.compareStr(a[columnKey], b[columnKey]);
                                };
                            }
                        }
                    })(key, this.customColumns[key]);
                }
            }
        }
    },
    queryCallback: function (remoteQuery, remoteOptions, resultQuery) {
        // function(remoteQuery, remoteOptions, resultQuery, resultOptions)
        this.inherited(arguments);
        if (this.customColumns != null &&
            remoteOptions != null &&
            "sort" in remoteOptions &&
            remoteOptions.sort.length == 1 &&
            remoteOptions.sort[0].attribute in this.customColumns &&
            "sort" in this.customColumns[remoteOptions.sort[0].attribute]) {
            delete remoteOptions["sort"];
            delete remoteOptions["start"];
            delete remoteOptions["count"];
        }
        if (this.customColumns != null && remoteQuery !== undefined && remoteQuery !== null) {
            for (var key in this.customColumns) {
                if (this.customColumns.hasOwnProperty(key)) {
                    var column = this.customColumns[key];
                    if ("filterTransform" in column && key in remoteQuery) {
                        var filter = column["filterTransform"](resultQuery[key]);
                        if (filter !== resultQuery[key]) {
                            delete remoteOptions["sort"];
                            delete remoteOptions["start"];
                            delete remoteOptions["count"];
                            delete remoteQuery[key];
                            resultQuery[key] = filter;
                            this["_useQueryEngine"] = true;
                        }
                    }
                    if ("get" in column && key in remoteQuery) {
                        delete remoteQuery[key];
                        this["_useQueryEngine"] = true;
                    }
                }
            }
        }
    },
    itemCallback: function(item) {
        if (this.customColumns != null) {
            for (var key in this.customColumns) {
                if ("get" in this.customColumns[key]) {
                    item[key] = this.customColumns[key].get(item);
                }
            }
        }
    },
    itemsCallback: function(items, query, options) {
        var list = items;
        if (("_useQueryEngine" in this) && this["_useQueryEngine"]) {
            var qe = this.queryEngine(this._memory._objectToQuery(query, options));
            var results  = qe(list);

            if (options.count && options.count != Infinity){
                var oldResults = results;
                results = results.slice(options.start, options.start + options.count);
                results.total = oldResults.length;
            } else {
                results.total = results.length;
            }

            return queryResults(results);
        } else if (this.customColumns != null &&
            options != null &&
            "sort" in options &&
            options.sort.length == 1 &&
            options.sort[0].attribute in this.customColumns &&
            "sort" in this.customColumns[options.sort[0].attribute]) {
            var ths = this;
            list.sort(function (a, b) {
                return ths.customColumns[options.sort[0].attribute].sort(a, b) * (options.sort[0].descending ? -1 : 1);
            });
            var count = "count" in options ? options["count"] : list.length;
            var start = "start" in options ? options["start"] : 0;
            if (count != list.length || start !== 0) {
                var result = [];
                for (var i = start; i < start + count && i < list.length; i++) {
                    result.push(list[i]);
                }
                delete options["start"];
                delete options["count"];
                list = result;
            }
            return list;
        } else {
            return list;
        }
    }
});

});