define([
    "dojo/_base/declare", "dojo/dom", "dojo/dom-construct", "dojo/dom-style",
    "dojo/dom-class", "dojo/dom-geometry",
    "aps/_Widget"],
function (declare, dom, domConstruct, domStyle, domClass, domGeom, Widget) {
    return declare([Widget], {

        //size limits
        minWidth: 20,
        minHeight: 20,
        maxWidth: 300,
        maxHeight: 300,

        //offset between crop region border and container border
        offset: 50,
        //size of frame border
        borderSize: 2,
        //the initial crop region size
        cropSize: 0,

        //whether to keep crop region as a square
        keepSquare: false,

        postCreate: function() {
            this.inherited(arguments);
            this.updateUI();
            this.connect(this.cropNode, 'onmousedown', '_onMouseDown');
            this.connect(document, 'onmouseup', '_onMouseUp');
            this.connect(document, 'onmousemove', '_onMouseMove');
            document.ondragstart = function() {
                return false;
            };
            dom.setSelectable(this.domNode, false);
            domClass.add(this.domNode, 'image-cropper');
            if (this.image) this.imageNode = domConstruct.create('img', {
                src: this.image
            }, this.domNode, 'first');
        },

        buildRendering: function() {
            this.inherited(arguments);
            this._archors = {};
            this._blockNodes = {};

            this.cropNode = domConstruct.create('div', {
                className: 'crop-node'
            }, this.domNode, 'last');

            var arr = ['lt', 't', 'rt', 'r', 'rb', 'b', 'lb', 'l'];
            var i;
            for (i = 0; i < 8; i++) {
                var img = domConstruct.create('img', {
                    className: arr[i]
                });
                img.src = this._blankGif;
                this.cropNode.appendChild(img);
                this._archors[arr[i]] = img;
            }

            arr = ['l', 't', 'r', 'b'];
            for (i = 0; i < 4; i++) {
                this._blockNodes[arr[i]] = domConstruct.create('div', {
                    className: 'block block-' + arr[i]
                }, this.domNode, 'last');
            }
        },

        setImage: function(url) {
            this.image = url;
            if (!this.imageNode) {
                this.imageNode = domConstruct.create('img', {
                    src: this.image
                }, this.domNode, 'first');
            }
            this.imageNode.src = url;
            if (!this.imageNode.offsetWidth) {
                var self = this;
                this.imageNode.onload = function() {
                    console.debug('img onload');
                    self.setSize(self.imageNode.offsetWidth, self.imageNode.offsetHeight);
                };
            } else {
                this.setSize(this.imageNode.offsetWidth, this.imageNode.offsetHeight);
            }
        },

        setSize: function(w, h) {
            console.debug('setting image cropper size: ', w, h);
            this.domNode.style.width = w + 'px';
            this.domNode.style.height = h + 'px';

            w = Math.min(w, this.maxWidth);
            h = Math.min(h, this.maxHeight);
            if (this.cropSize) {
                w = Math.min(w, this.cropSize);
                h = Math.min(h, this.cropSize);
            }
            domStyle.set(this.cropNode, {
                width: Math.min(w, this.cropSize) - this.borderSize + 'px',
                height: Math.min(h, this.cropSize) - this.borderSize + 'px'
            });

            domStyle.set(this.cropNode, {
                left: '0px',
                top: '0px'
            });
            this.posArchors();
            this.posBlocks();
            this.onChange(this.getInfo());
        },

        setCrop: function(t, l, w, h) {
            domStyle.set(this.cropNode, {
                top: t + 'px',
                left: l + 'px',
                width: w - this.borderSize + 'px',
                height: h - this.borderSize + 'px'
            });
            this.posArchors();
            this.posBlocks();
            this.onChange(this.getInfo());
        },

        updateUI: function() {
            this.posArchors();
            this.posBlocks();

        },

        posArchors: function() {
            var a = this._archors,
                w = this.cropNode.offsetWidth,
                h = this.cropNode.offsetHeight;

            a.t.style.left = a.b.style.left = w / 2 - this.borderSize * 2 + 'px';
            a.l.style.top = a.r.style.top = h / 2 - this.borderSize * 2 + 'px';
        },

        posBlocks: function() {
            var b = this._blockNodes;
            var l = parseInt(this.cropNode.style.left || domStyle.get(this.cropNode, 'left'), 10);
            var t = parseInt(this.cropNode.style.top || domStyle.get(this.cropNode, 'top'), 10);
            var frameW = this.cropNode.offsetWidth;
            var imgW = this.domNode.offsetWidth;
            var frameH = this.cropNode.offsetHeight;
            var imgH = this.domNode.offsetHeight;

            b.t.style.height = b.l.style.top = b.r.style.top = t + 'px';

            b.l.style.height = b.r.style.height = frameH + 'px';
            b.l.style.width = l + 'px';

            b.r.style.width = imgW - frameW - l + 'px';
            b.b.style.height = imgH - frameH - t + 'px';
        },

        _onMouseDown: function(e) {
            this.dragging = e.target == this.cropNode ? 'move' : e.target.className;
            var pos = domGeom.position(this.cropNode);
            var imgPos = domGeom.position(this.domNode);
            // keep the screen coords user started to move from (x, y)
            // and actual position of the crop frame on image node (t, l)
            this.startedPos = {
                x: e.pageX,
                y: e.pageY,
                h: pos.h - this.borderSize,
                w: pos.w - this.borderSize,
                l: pos.x - imgPos.x,
                t: pos.y - imgPos.y
            };
            var c = domStyle.get(e.target, 'cursor');
            domStyle.set(document.body, {
                cursor: c
            });
            domStyle.set(this.cropNode, {
                cursor: c
            });

        },

        _onMouseUp: function() {
            this.dragging = false;
            domStyle.set(document.body, {
                cursor: 'default'
            });
            domStyle.set(this.cropNode, {
                cursor: 'move'
            });
            this.onDone(this.getInfo());
        },
        getInfo: function() {
            return {
                width: this.cropNode.offsetWidth,
                height: this.cropNode.offsetHeight,
                left: parseInt(this.cropNode.style.left || domStyle.get(this.cropNode, 'left'), 10),
                top: parseInt(this.cropNode.style.top || domStyle.get(this.cropNode, 'top'), 10),
                containerWidth: this.domNode.offsetWidth,
                containerHeight: this.domNode.offsetHeight
            };
        },
        _onMouseMove: function(e) {
            if (!this.dragging) return;

            if (this.dragging == 'move') this.doMove(e);
            else this.doResize(e);
            this.updateUI();

            this.onChange(this.getInfo());
        },
        doMove: function(e) {
            //console.debug('doing move',e);
            var s = this.cropNode.style,
                p0 = this.startedPos;
            var l = p0.l + e.pageX - p0.x;
            var t = p0.t + e.pageY - p0.y;
            if (l < 0) l = 0;
            if (t < 0) t = 0;
            var maxL = this.domNode.offsetWidth - this.cropNode.offsetWidth;
            var maxT = this.domNode.offsetHeight - this.cropNode.offsetHeight;
            if (l > maxL) l = maxL;
            if (t > maxT) t = maxT;
            s.left = l + 'px';
            s.top = t + 'px';

        },
        onChange: function() {
            //Event:
            //    When the cropping size is changed.
        },
        onDone: function() {
            //Event:
            //    When mouseup.
        },
        doResize: function(e) {
            var mode = this.dragging,
                style = this.cropNode.style,
                p0 = this.startedPos;
            //delta x and delta y
            var dx = e.pageX - p0.x,
                dy = e.pageY - p0.y;

            if (this.keepSquare) {
                if (mode == 'l') {
                    dy = dx;
                    if (p0.l + dx < 0) dx = dy = -p0.l;
                    if (p0.t + dy < 0) dx = dy = -p0.t;
                    mode = 'lt';
                } else if (mode == 'r') {
                    dy = dx;
                    mode = 'rb';
                } else if (mode == 'b') {
                    dx = dy;
                    mode = 'rb';
                } else if (mode == 'lt') {
                    dx = dy = Math.abs(dx) > Math.abs(dy) ? dx : dy;
                    if (p0.l + dx < 0) dx = dy = -p0.l;
                    if (p0.t + dy < 0) dx = dy = -p0.t;
                } else if (mode == 'lb') {
                    dy = -dx;
                    if (p0.l + dx < 0) {
                        dx = -p0.l;
                        dy = p0.l;
                    }
                } else if (mode == 'rt' || mode == 't') {
                    dx = -dy;
                    mode = 'rt';
                    if (p0.t + dy < 0) {
                        dy = -p0.t;
                        dx = -dy;
                    }
                }
            }
            if (/l/.test(mode)) {
                dx = Math.min(dx, p0.w - this.minWidth);
                if (p0.l + dx >= 0) {
                    style.left = p0.l + dx + 'px';
                    style.width = p0.w - dx + 'px';
                } else {
                    style.left = 0;
                    style.width = p0.l + p0.w + 'px';
                }
            }
            if (/t/.test(mode)) {
                dy = Math.min(dy, p0.h - this.minHeight);
                if (p0.t + dy >= 0) {
                    style.top = p0.t + dy + 'px';
                    style.height = p0.h - dy + 'px';
                } else {
                    style.top = 0;
                    style.height = p0.t + p0.h + 'px';
                }
            }
            if (/r/.test(mode)) {
                dx = Math.max(dx, this.minWidth - p0.w);
                if (p0.l + p0.w + dx <= this.domNode.offsetWidth) {
                    style.width = p0.w + dx + 'px';
                } else {
                    style.width = this.domNode.offsetWidth - p0.l - this.borderSize + 'px';
                }
            }
            if (/b/.test(mode)) {
                dy = Math.max(dy, this.minHeight - p0.h);
                if (p0.t + p0.h + dy <= this.domNode.offsetHeight) {
                    style.height = p0.h + dy + 'px';
                } else {
                    style.height = this.domNode.offsetHeight - p0.t - this.borderSize + 'px';
                }
            }

            if (parseInt(style.width, 10) > this.maxWidth - this.borderSize) {
                style.width = this.maxWidth - this.borderSize + 'px';
            }
            if (parseInt(style.height, 10) > this.maxHeight - this.borderSize) {
                style.height = this.maxHeight - this.borderSize + 'px';
            }

            if (this.keepSquare) {
                var min = Math.min(parseInt(style.width, 10), parseInt(style.height, 10));
                style.height = style.width = min + 'px';
            }
        }
    });
});