define([
    "dojo/_base/declare",
    "dojo/on",
    "aps/_Widget",
    "dijit/_TemplatedMixin",
    "dijit/registry",
    "dojo/dom-construct"
], function (declare, on, _Widget, _Templated, registry, domConstruct) {
    return declare([_Widget, _Templated], {
        // summary:
        // 		Widget represents a keypad slector.


        // baseClass: [private] string
        // 		Root CSS class of the widget (ex: dijitTextBox),
        // 		used to construct CSS classes to indicate widget state.
        baseClass: "voiceKeypadSelector",

        // selectedKey: string
        //      A selected phone key.
        selectedKey: "",

        // disabledKeys: array
        //      Array of disabled keys.
        disabledKeys: [],

        // keys: [optional] array
        //      Array of shown keys. Default: 1, 2, 3, 4, 5, 6, 7, 8, 9
        //      Cannot be changed after creating.
        keys: ["1", "2", "3", "4", "5", "6", "7", "8", "9"],

        // templateString: [private] string
        // 		Template string
        templateString: '<div></div>',

        // _isReady: [private] boolean
        //      Indicates whether the keypad is ready
        _isReady: false,

        _setDisabledKeysAttr: function (val) {
            var self = this;

            if (self._isReady) {
                self.keys.forEach(function (key) {
                    var className = val.indexOf(key) > -1 ? "phoneKey disabled" : key == self.selectedKey ? "phoneKey active" : "phoneKey";
                    document.getElementById(self.id + "_" + key).className = className;
                });
            }

            this._set("disabledKeys", val);
        },

        _setSelectedKeyAttr: function (val){
            if (this.selectedKey != val) {
                if (this.selectedKey) {
                    var activeKey = document.getElementById(this.id + "_" + this.selectedKey);
                    activeKey.className = "phoneKey";
                }

                if (val) {
                    var newActiveKey = document.getElementById(this.id + "_" + val);
                    newActiveKey.className = "phoneKey active";
                }
                this._set("selectedKey", val);
            }
        },

        _onKeyClick: function (e) {
            // summary:
            //      processes a cick on key.
            // tags:
            // 		private

            var keypadSelector = registry.getEnclosingWidget(e.target);
            var key = e.target.innerHTML;
            if (keypadSelector.selectedKey != key && keypadSelector.disabledKeys.indexOf(key) < 0) {
                keypadSelector.set("selectedKey", key);
            }
        },

        postCreate: function () {
            // summary:
            //		Post create processing.
            // tags:
            // 		Public

            var self = this;
            self.inherited(arguments);

            self.keys.forEach(function (key) {
                var className = self.disabledKeys.indexOf(key) > -1 ? "phoneKey disabled" : key == self.selectedKey ? "phoneKey active" : "phoneKey";
                var item = domConstruct.create("b", { "class": className, innerHTML: key, id: self.id + "_" + key }, self.domNode);
                self.own(
                    on(item, "click", self._onKeyClick)
                );
            });

            self._isReady = true;
        }
    });
});