define([
    "dojo/_base/declare",
    "dojo/on",
    "aps/Button",
    "aps/i18n",
    "aps/Container",
    "aps/Tooltip",
    "aps/_Widget",
    "aps/_Container",
    "dijit/_TemplatedMixin",
    "dijit/registry",
	"dojo/_base/array",
    "dojo/_base/lang",
    "dojo/dom-construct",
    "dojo/query"
], function (
    declare,
    on,
    Button,
    _,
    Container,
    Tooltip,
    _Widget,
    _Container,
    _Templated,
    registry,
    arr,
    lang,
    domConstruct,
    query
) {
    return declare([_Widget, _Container, _Templated], {
        // summary:
        // 		Widget represents a modal window.
        // description:
        // 		This widget should be used ONLY in VoiceMenu.


        // baseClass: [private] string
        // 		Root CSS class of the widget (ex: dijitTextBox),
        // 		used to construct CSS classes to indicate widget state.
        baseClass: "lyncModalWindow",

        // controls:
        controls: {
            submit: { label: _("OK"), visible: true },
            cancel: { label: _("Cancel"), visible: true }
        },

        // content: Container
        //      contains the dialog content.
        content: null,

        // model: object
        //      object defining data to view
        model: null,

        // maxHeight: integer|null
        //      if specified, defines the maximum window height.
        maxHeight: null,

        // onSubmit: function
        //      processes pressing Submit button and return true if successed.
        //      may returns deferred boolean result.
        onSubmit: null,

        // onCancel: function
        //      processes canceling window. Calls when Cancel button is clicked or when window is escaped.
        onCancel: null,

        // templateString: [private] string
        // 		Template string
        templateString: '<div> <div class="voiceDialog"> <div class="lyncModalWindowContent"></div> <div class="btns-box"><div class="box-area"></div></div> </div></div>',

        // _buttons: [private] array
        //      contains a control buttons
        _buttons: [],

        // _overlayArea: [private] node
        //      Represent an overlay area.
        _overlayArea: null,

        _setMaxHeightAttr: function (val) {
            var self = this;
            if (val != self.maxHeight) {
                self.domNode.style["maxHeight"] = (val && val > 0) ? val + "px" : "";
                self._set("maxHeight", val);
            }
        },

        show: function () {
            // summary:
            //      Shows modal window
            // tags:
            // 		public
            var self = this;
            self._overlayArea.style.display = "block";
            self.set("visible", true);
        },

        hide: function () {
            // summary:
            //      Hides modal window
            // tags:
            // 		public
            this._overlayArea.style.display = "none";
            this.set("visible", false);
        },

        addChild: function (child, insertIndex) {
            // summary:
            //		Makes the given widget a child of window content.
            // tags:
            // 		public
            // child: Widget
            // 		Child widget
            // insertIndex: Number || String
            // 		Position child in the parent widget
            this.content.addChild(child, insertIndex);
        },

        removeChild: function (/*Widget|int*/ widget) {
            // summary:
            //		Removes the passed widget instance from the window content.
            // 		You can also pass in an integer indicating
            //		the index within the container to remove (ie, removeChild(5) removes the sixth widget).
            this.content.removeChild(widget);
        },

        postCreate: function () {
            // summary:
            //		Post create processing.
            // tags:
            // 		Public

            var self = this;
            self.inherited(arguments);

            var buttonArea = query(".box-area", this.domNode)[0];
            Object.keys(self.controls).forEach(function (controlKey) {
                var control = self.controls[controlKey];
                var processOnClick = null;
                var className = "btn";
                switch (controlKey) {
                    case "submit":
                        processOnClick = self.onSubmit;
                        className = "btn action";
                        break;
                    case "cancel":
                        processOnClick = self.onCancel;
                        break;
                }
                if (control.visible) {
                    var button = new Button({
                        id: self.id + "_" + controlKey,
                        title: control.label,
                        "class": className,
                        processOnClick: processOnClick,
                        onClick: function () {
                                this.cancel();
                            if (!this.processOnClick) {
                                self.hide();
                            } else {
                                var res = this.processOnClick();
                                if (typeof res == "boolean") {
                                    if (res) {
                                        self.hide();
                                    }
                                } else {
                                    res.then(function (isOkToClose) {
                                        if (isOkToClose) {
                                            self.hide();
                                        }
                                    });
                                }
                            }
                        }
                    });

                    self._buttons.push(button);
                    domConstruct.place(button.domNode, buttonArea);
                }
            });

            if (self.content == null) {
                self.content = new Container({
                    id: self.id + "_" + "content"
                });
            }

            domConstruct.place(self.content.domNode, query(".lyncModalWindowContent", this.domNode)[0]);

            var pageContainer = registry.byId("pageContainer");
            self._overlayArea = domConstruct.create("div", { "class": "voiceDialogOverlay" }, pageContainer.domNode, "first");
            self.own(on(self._overlayArea, "click", lang.hitch(self, function(){
                if (!this.onCancel || this.onCancel()){
                    this.hide();
                }
            })));

            self.hide();
        },

        destroy: function () {
            var self = this;
            self.inherited(arguments);

            if (this._buttons) {
                this._buttons.forEach(function (button) {
                    button.destroy();
                });
            }

            if (this.content) {
                this.content.destroy();
            }
        }
    });
});