define([
        "dojo/_base/declare",
        "aps/i18n",
        "aps/TextArea",
        "aps/Tooltip",
        "dijit/registry",
        "dojo/on",
        "./utils.js"
], function (
    declare,
    _,
    TextArea,
    Tooltip,
    registry,
    on,
    utils
) {
    return declare([TextArea], {
        // summary:
        //	Customized TextArea for Text-To-Speech text.

        _onKeypress: function (e) {
            /// <summary>Ignores chevrons onKeypress and shows the corresponding Tooltip.</summary>
            if (e.charCode == 60 || e.charCode == 62) {
                e.preventDefault();
                Tooltip.show(_("Angle brackets ('<' and '>') are not allowed."), registry.getEnclosingWidget(e.target).domNode);
            }
        },

        _onPaste: function (e) {
            /// <summary>Removes angle brackets (chevrons) pasted text onPaste and shows the corresponding Tooltip.</summary>

            var copiedData = '';
            if (e.clipboardData && e.clipboardData.getData) {
                copiedData = e.clipboardData.getData('Text');
            } else if (window.clipboardData) {
                copiedData = window.clipboardData.getData('Text');
            }

            if (!copiedData) {
                return;
            }

            e.preventDefault();

            var trgtWidget = registry.getEnclosingWidget(e.target);
            if (trgtWidget.disabled) {
                return;
            }

            if (copiedData.match(/[<>]/gm)) {
                Tooltip.show(_("Angle brackets ('<' and '>') are not allowed and removed from the pasted text."), trgtWidget.domNode);
            }

            copiedData = copiedData.replace(/[<>]/gm, '');
            var selStart = e.target.selectionStart,
                selEnd = e.target.selectionEnd;
            var oldValue = e.target.value;
            e.target.value = oldValue.substring(0, selStart).concat(copiedData, oldValue.substring(selEnd));
            e.target.selectionStart = selStart;
            copiedData = copiedData.replace(/\r\n/gm, '\n'); // in order to properly count the selection length
            e.target.selectionEnd = selStart + copiedData.length;
            trgtWidget.set('value', oldValue.substring(0, selStart).concat(copiedData, oldValue.substring(selEnd)));
        },

        _onChange: function (e) {
            /// <summary>Trims and validates TextToSpeech value onChange event.</summary>
            e.target.value = utils.trimStr(e.target.value);
            if (!e.target.value) {
                Tooltip.show(_("This value is required."), registry.getEnclosingWidget(e.target).domNode);
            } else if (e.target.value.match(/[<>]/gm)) {
                Tooltip.show(_("Angle brackets ('<' and '>') are not allowed and removed from the pasted text."), registry.getEnclosingWidget(e.target).domNode);
                e.target.value = e.target.value.replace(/[<>]/gm, '');
            }
        },

        _hideTooltip: function (e) {
            Tooltip.hide(registry.getEnclosingWidget(e.target).domNode);
        },

        postCreate: function () {
            this.inherited(arguments);
            this.own(
                on(this.focusNode, "keypress", this._onKeypress),
                on(this.focusNode, "paste", this._onPaste),
                on(this.focusNode, "change", this._onChange),
                on(this.focusNode, "mouseout", this._hideTooltip),
                on(this.focusNode, "blur", this._hideTooltip),
                on(this.focusNode, "click", this._hideTooltip)
            );
        }
    });

});