var packageLocation = location.pathname.replace(/\/[^/]+$/, '/');

define([
        "aps/Gauge",
        "aps/Grid",
        "aps/i18n",
        "aps/Memory",
        "aps/Output",
        "dojo/_base/declare",
        "dojo/json",
        "dojo/on",
        "dojo/text!" + packageLocation + "/skuTypes.json",
        "dojox/html/entities",
        "./utils.js"
], function(
        Gauge,
        Grid,
        _,
        Memory,
        Output,
        declare,
        JSON,
        on,
        skuTypes,
        entities,
        utils
) {
    return declare([Grid], {

        skuStore: new Memory(JSON.parse(skuTypes)),

        value : null,
        profiles: null,

        minAvailableCount: 1,   // required available licenses count

        voice: false,           // get-only parameter

        selectionMode: 'single',
        showPaging: false,
        rowsPerPage: Infinity,
        noEntriesFoundText: _('There are no available licenses in your account. In order to assign Lync to new users, please order Lync licenses first.'),
        "class": "notSortableBold",

        _gridCreated: false,

        postMixInProperties: function() {
            var self = this;
            self.inherited(arguments);

            var isVoiceEnabled = self.profiles.some(function(profile) { return self.skuStore.get(profile.service.sku).voice;});

            self.columns = [
                {
                    'name': _('License'),
                    'field': 'service.name',
                    'renderCell': function(row, val, td) {
                        if (self.selectionMode != 'none') {
                            var profile = row;
                            var inputElem = td.parentNode.children[0].children[0];

                            var disabled = self._isProfileDisabled(profile);
                            if (!disabled) {
                                on(inputElem.parentNode.parentNode, "click", function() {
                                    inputElem.click();
                                });
                            }
                            inputElem.disabled = disabled;
                        }

                        return utils.buildStatusOutput(row.resource.limit == "unlimited" || row.resource.availableCount > 0 ? 'hint-ok' : "hint-failed", entities.encode(val));
                    },
                    'sortable': false
                },
                // {
                //     'name': '',
                //     'field': 'service.description',
                //     'sortable': false
                // },
                {
                    'name': _('Features'),
                    'renderCell': function (row) {
                        var sku = self.skuStore.get(row.service.sku);
                        var features = "<i class='icon'><img src='images/16/online-chat.png'></i> " + entities.encode(_("Instant Messaging & Presence"));
                        var meetingIcon = "<i class='icon'><img src='images/16/online-meeting.png'></i> " + _("Online Meetings");
                        if (isVoiceEnabled) {
                            var voiceIcon = "<i class='icon'><img src='images/16/phone-calls.png'></i> " + _("Phone Calls");
                            if (sku.voice) {
                                features += " &nbsp; " + voiceIcon;
                            } else {
                                features += " &nbsp; <span class='hint'>" + voiceIcon.replace(/phone-calls.png/, "phone-calls-disabled.png") + "</span>";
                            }
                        }
                        if (sku.conferencing) {
                            features += " &nbsp; " + meetingIcon;
                        } else {
                            features += " &nbsp; <span class='hint'>" + meetingIcon.replace(/online-meeting.png/, "online-meeting-disabled.png") + "</span>";
                        }
                        return new Output({content: features, escapeHTML: false});
                    },
                    'sortable': false
                },
                {
                    'name': _('Usage'),
                    'renderCell': function (row) {
                        if (row.resource.limit == "unlimited") {
                            return new Output({
                                content: _("Used __usage__ Licenses", { usage: "${usage}" } ),
                                usage: row.resource.usage
                            });
                        } else {
                            var maxValue = row.resource.limit;
                            var classesMap = {};
                            classesMap[maxValue * 0.5] = "warn";
                            classesMap[maxValue * 0.85] = "over";
                            return new Gauge({
                                legend: _("Used __usage__ of __maxValue__ Licenses", { usage: row.resource.usage, maxValue: maxValue } ),
                                minimum: 0,
                                maximum: maxValue,
                                value: row.resource.usage,
                                'classesMap': classesMap,
                                width: "auto"
                            });
                        }
                    },
                    'sortable': false
                }];

            if (self.profiles.some(function(profile) { return profile.resource.availableCount != "unlimited"; })) {
                self.columns.push({
                    'name': _('Remaining'),
                    'renderCell': function(row) {
                        if (row.resource.limit == "unlimited") {
                            return _("Unlimited");
                        } else {
                            var availableText = _("__count__ Licenses", { count: row.resource.availableCount });
                            if (row.resource.availableCount === 0) {
                                availableText = "<span class='error-hint'>" + availableText + "</span>";
                            }
                            return new Output({content: availableText, escapeHTML: false});
                        }
                    },
                    'sortable': false
                });
            }

            if (self.store == null) {
                self.profiles.sort(function (a, b) { return a.service.sku > b.service.sku; });
                self.store = new Memory({ data: self.profiles, idProperty: "aps.id" });
            }
        },

        postCreate: function () {
            var self = this;
            self.inherited(arguments);

            self.selectionArray.watchElements(function(){
                if (self.selectionArray.length > 0) {
                    self.set("value", self.selectionArray[0]);
                }
            });

            self._updateVoice();
        },

        startup: function() {
            var self = this;
            self.inherited(arguments);

            //self.set("showPaging", false);

            self._gridCreated = true;

            if (self.value == null) {
                for (var i=0; i<self.profiles.length; i++) {
                    if (! self._isProfileDisabled(self.profiles[i])) {
                        self.set("value", self.profiles[i].aps.id);
                        break;
                    }
                }
            } else {
                self.set("value", self.value);
            }
        },

        _setValueAttr: function (value) {
            var self = this;
            self.inherited(arguments);
            if (self.value != value || (self._gridCreated && self.selectionArray != null && self.selectionArray.length === 0)) {
                self._set("value", value);
                self._updateVoice();

                if (self._gridCreated) {
                    self.selectionArray.splice(0, 1, value);
                }
            }
        },

        _getValueAttr: function () {
            var self = this;
            return self.value;
        },

        _setVoiceAttr: function (voice) {
            var self = this;
            self.inherited(arguments);

            if (self.voice != voice) {
                self._set("voice", voice);
            }
        },

        _getVoiceAttr: function () {
            var self = this;
            return self.voice;
        },

        getUserProfile: function(id) {
            var self = this;
            return self.store.get(id == null ? self.value : id);
        },

        _updateVoice: function() {
            var self = this;
            if (self.value == null) {
                self._set("voice", false);
            } else {
                var profile = self.getUserProfile(self.value).service;
                self._set("voice", profile != null && self.skuStore.get(profile.sku).voice);
            }
        },

        _isProfileDisabled: function(profile) {
            var self = this;
            return !(profile.resource.limit == 'unlimited' || profile.resource.availableCount >= self.minAvailableCount);
        }
    });
});