define([
    "dojo/_base/declare",
    "aps/i18n",
    "aps/Select",
    "aps/ResourceStore",
    "./displayError.js",
    "./utils.js"
], function (
    declare,
    _,
    Select,
    Store,
    displayError,
    utils
) {
    //"use strict";
    return declare([Select], {

        // _defaultOption: [private]
        _defaultOption: { label: _("-- Select User --"), value: null, login: null, disabled: true },

        // organizationId: string
        //      id of organization resource.
        //      if it's not specified, tries to get from aps.context.vars.organization
        organizationId: null,

        // requireEnterpriseVoice: bool
        //      a value indicating whether to list only users with Enterprise Voice.
        //      default: false
        requireEnterpriseVoice: false,

        // defaultOption: { "label": <user name>, "value" : <user id>, "login": <user login> [, "selected": bool] [, "disabled": bool]}
        //      An item to be shown while loading users
        defaultOption: null,

        getSelectedUserLogin: function() {
            var self = this;
            if (self.value) {
                var selectedOption = self.getOption(self.value);
                if (selectedOption && selectedOption.length > 0) {
                    return selectedOption[0].login;
                }
            }

            return null;
        },

        postCreate: function() {
            var self = this;
            self.inherited(arguments);

            self.set("options", [self.defaultOption || self._defaultOption, { label: _("Loading users. Please, wait..."), value: "loading", disabled: true, login: null }]);

            if (!self.organizationId) {
                self.organizationId = aps.context.vars.organization.aps.id;
            }

            new Store({
                target: "/aps/2/resources/" + self.organizationId + "/userLicenses?eq(aps.status,aps%3Aready),sort(+displayName)" + (self.requireEnterpriseVoice ? ",eq(enterpriseVoiceEnabled,true)" : "")
            }).query().then(function(users) {
                if (users && users.length > 0) {
                    var options = [self._defaultOption];
                    if (!self.value) {
                        options[0].selected = true;
                    }

                    var containsDefault = !(self.defaultOption && self.defaultOption.value);
                    users.forEach(function (user) {
                        options.push({
                            label: user.displayName,
                            value: user.aps.id,
                            selected: user.aps.id === self.value,
                            login: user.login
                        });
                        
                        if (!containsDefault && user.aps.id == self.defaultOption.value) {
                            containsDefault = true;
                        }
                    });

                    if (!containsDefault) {
                        self.defaultOption.selected = self.defaultOption.value == self.value;
                        self.defaultOption.disabled = true;
                        options.push(self.defaultOption);
                        options.sort(function (a, b) {
                            if (a.value == self._defaultOption.value) {
                                return -1;
                            }
                            
                            if (b.value == self._defaultOption.value) {
                                return 1;
                            }

                            return utils.compareStr(a.label, b.label);
                        });
                    }

                    self.set("options", options);
                } else {
                    self.set("options", [self._defaultOption, { labe: _("No users available."), value: "noUsers", "login": null, disabled: true }]);
                    self.set("value", null);
                }
            }, function(err) {
                displayError(err);
            });
        }
    });
});