define([
    "dojo/_base/declare",
    "aps/_Widget",
    "aps/_Container",
    "dijit/_TemplatedMixin",
    "aps/Container",
    "aps/i18n",
    "aps/Message",
    "aps/MultiSelect",
    "aps/Output",
    "aps/RadioButton",
    "aps/ResourceStore",
    "aps/Select",
    "aps/TextBox",
    "aps/ToolbarButton",
    "aps/Tooltip",
    "aps/xhr",
    "dijit/registry",
    "dojo/dom-construct",
    "dojo/dom-geometry",
    "dojo/_base/array",
    "dojo/_base/lang",
    "dojo/query",
    "dojo/mouse",
    "dojo/on",
    "dojo/when",
    "dojo/Deferred",
    "dojo/json",
    "dojox/mvc/at",
    "dojox/mvc/getStateful",
    "dojox/mvc/getPlainValue",
    "./VoiceMenuItem.js",
    "./VoiceMenuMessage.js",
    "./KeypadSelector.js",
    "./ModalWindow.js",
    "./PhoneNumberSelector.js",
    "./utils.js"
], function(
    declare,
    _Widget,
    _Container,
    _Templated,
    Container,
    _,
    Message,
    MultiSelect,
    Output,
    RadioButton,
    Store,
    Select,
    TextBox,
    ToolbarButton,
    Tooltip,
    xhr,
    registry,
    domConstruct,
    domGeometry,
    arr,
    lang,
    query,
    mouse,
    on,
    when,
    Deferred,
    json,
    at,
    getStateful,
    getPlainValue,
    VoiceMenuItem,
    VoiceMenuMessage,
    KeypadSelector,
    ModalWindow,
    PhoneNumberSelector,
    utils
) {
    return declare([_Widget, _Container, _Templated], {
        // summary:
        //	Define container containing Call Group's menu items.

        _new: "new",

        _isQuestion: function(value) { return value == window.constants.toQuestionTxt; },
        _isQueue: function(value) { return value == window.constants.toQueueTxt; },
        _isNewGroup: function(value) { return value == 'new'; },
        _isNot: function (value) { return !value; },

        // baseClass: [private] string
        // 		Root CSS class of the widget (ex: dijitTextBox),
        // 		used to construct CSS classes to indicate widget state.
        baseClass: "voiceTree",

        // menu: object
        //      call group menu object
        menu: null,

        // phoneNumber: object {displayNumber: string, apsId: string}
        //      a call group phone number.
        phoneNumber: null,

        // existingQueues: aps/Memory
        //      Represents a store of all available user groups
        existingQueues: null,

        // agents: store (i.e. aps/Memory)
        //      Represents a store of all available lync users that can be selected as group agents.
        lyncUsers: null,

        // organizationId: string
        //      Organization id is required to retrieve phone numbers.
        organizationId: null,

        // greetingSip: string
        //      Primary sip uri of call group.
        greetingSip: "",

        // isNew: bool
        //      Value indicating whether this greeting menu is a new one or this is an existing greeting.
        isNew: false,

        // userGroupLimit: int, "unlimitted"
        //      A value representing the count of user groups that may be created.
        userGroupLimit: 0,

        // templateString: [private] string
        // 		Template string
        templateString: '<ul><li><span class="newVoiceMenuWarning"></span></li><li><span class="voiceMenuItemError"></span><span class="arrowDown"></span><a class="voicePhone" data-dojo-attach-event="ondijitclick:_onPhoneClick"><p>${_phoneNumberTitle} <span class="voiceMenuDisplayPhone">${phoneNumber.displayNumber}</span></p></a></li></ul>',

        // _onPhoneClick: function
        //      process click on phone number item
        _onPhoneClick: function () {
            var self = this;
            self._phoneNumberWindow.show();

            var el = query(".voicePhone", self.domNode)[0];

            if (el.classList) {
                el.classList.add("active");
            } else {
                el.className = "active";
            }

            // positioning
            var itemPosition = domGeometry.position(el);
            self._phoneNumberWindow.domNode.style["top"] = itemPosition.y + "px";
            self._phoneNumberWindow.domNode.style["left"] = (itemPosition.x + 40) + "px";

            /*
            var voiceMenuPosition = domGeometry.position(self.domNode);
            console.log(voiceMenuPosition);
            var position = domGeometry.position(self._phoneNumberWindow.domNode);
            console.log(position);
            if (position.y + maxHeight > voiceMenuPosition.h) {
                self.domNode.style["minHeight"] = position.y + position.h + "px";
            }
            */
        },

        // localized item text strings.
        _phoneNumberTitle: _("Get Inbound Call to"),

        // _menuItem: [private] VoiceMenuItem
        //      widget represent voice menu
        _menuItem: null,

        // _phoneNumberWindow: [private] ModalWindow
        //      Modal window to edit phone number.
        _phoneNumberWindow: null,

        // _voiceItemWindow: [private] ModalWindow
        //      Modal window to edit voice items.
        _menuItemWindow: null,

        // _newUserGroupCount: int
        //      A count of defined new user groups.
        _newUserGroupCount: 0,

        _definePhoneNumberWindow: function() {
            // summary:
            //      defines a modal window to edit phone number
            // tags:
            // 		private

            var self = this;

            self._phoneNumberWindow = new ModalWindow({
                id: self.id + "_" + "phoneWindow",
                model: getStateful({
                    apsId: self.phoneNumber.apsId
                }),
                onCancel: function() {
                    self.domNode.style["minHeight"] = "";
                    query(".newVoiceMenuWarning", self.domNode).style("display", "none");
                    query(".active", self.domNode).removeClass("active");
                    return true;
                },
                onSubmit: function() {
                    var phWindow = registry.getEnclosingWidget(this.domNode.parentNode);
                    self.phoneNumber.set("apsId", phWindow.model.apsId);
                    var phone = registry.byId(phWindow.id + "_selector").getNumber(phWindow.model.apsId);
                    self.phoneNumber.set("displayNumber", phone.displayNumber);
                    self.domNode.style["minHeight"] = "";
                    query(".newVoiceMenuWarning", self.domNode).style("display", "none");
                    query(".active", self.domNode).removeClass("active");
                    return true;
                }
            });

            var phoneSelector = new PhoneNumberSelector({
                organizationId: self.organizationId,
                title: _("Phone Number"),
                value: at(self._phoneNumberWindow.model, "apsId"),
                id: self._phoneNumberWindow.id + "_selector",
                phoneNumberOwner: self.greetingSip
            });
            phoneSelector.waitForLoad.then(function (hasNumbers) {
                if (!hasNumbers) {
                    var msg;
                    self.own(
                        msg = new Message({
                            description: "You do not have available phone numbers. <a href='javascript:aps.apsc.gotoView(\"phoneNumberBlocks.list\")'>Request</a> an additional Phone number block or release some of your existing phone numbers to proceed.",
                            type: "warning",
                            closeable: false
                        })
                    );
                    domConstruct.place(msg.domNode, query(".voiceMenuItemError", self.domNode)[0]);
                    aps.app.onSubmit = function () {
                        query(".msg-warning", self.domNode).replaceClass("msg-error", "msg-warning");
                        aps.apsc.cancelProcessing();
                    };

                    return;
                }

                self.phoneNumber.set("apsId", phoneSelector.value);
                self.phoneNumber.set("displayNumber", phoneSelector.getNumber(phoneSelector.value).displayNumber);
            });

            self._phoneNumberWindow.addChild(phoneSelector);
            self.addChild(self._phoneNumberWindow);
        },

        _defineMenuItemWindow: function() {
            // summary:
            //      defines a modal window to edit menu items
            // tags:
            // 		private

            var self = this;
            self._menuItemWindow = new ModalWindow({
                id: self.id + "_" + "menuItemWindow",
                model: getStateful({
                    selectedKey: "",
                    disabledKeys: [],
                    level: 1,
                    action: window.constants.toQueueTxt,
                    question: {
                        messageType: window.constants.textToSpeechTxt,
                        textToSpeech: "",
                        audioFile: {
                            name: null,
                            identity: null,
                            _selectedFile: null,
                            _agreementConfirmed: false
                        }
                    },
                    agentQueueId: "new",
                    _groupName: "",
                    _groupUsers: [],
                    _isNewGroupAllowed: false
                }),
                _hideTooltips: function() {
                    Tooltip.hide(registry.byId(this.id + "_groupId").domNode);
                    Tooltip.hide(registry.byId(this.id + "_groupUsers").domNode);
                    registry.byId(this.id + "_message")._hideTooltips();
                },
                onCancel: function() {
                    self.domNode.style["minHeight"] = "";
                    query(".newVoiceMenuWarning", self.domNode).style("display", "none");
                    query(".active", self.domNode).removeClass("active");
                    self._updateUserGroupCount();
                    self._menuItemWindow.model._groupUsers.set("length", 0);
                    self._menuItemWindow._hideTooltips();
                    return true;
                },
                onSubmit: function() {
                    var miWindow = registry.getEnclosingWidget(this.domNode.parentNode);
                    miWindow._hideTooltips();
                    var currentModel = getPlainValue(miWindow.model);
                    var validateMenuItem = function() {
                        var validationResult = new Deferred();
                        var isValid = true;
                        if (currentModel.action == window.constants.toQueueTxt) {
                            if (currentModel.level === 0) {
                                isValid = registry.byId(miWindow.id + "_message").validateMessage();
                            }

                            if (currentModel.agentQueueId == "new") {
                                if (!currentModel._isNewGroupAllowed) {
                                    validationResult.resolve(false);
                                    return validationResult;
                                }

                                var groupNameTextbox = registry.byId(miWindow.id + "_groupName");
                                isValid = isValid && groupNameTextbox.validate();
                                if (!isValid) {
                                    groupNameTextbox.focus();
                                }

                                if (!currentModel._groupUsers || currentModel._groupUsers.length === 0) {
                                    Tooltip.show(_("Group members are not selected."), registry.byId(miWindow.id + "_groupUsers").domNode);
                                    isValid = false;
                                }
                            } else {
                                var selectedQueue = self.existingQueues.get(currentModel.agentQueueId);
                                currentModel._groupName = selectedQueue.name;
                                var noAgentsMsg = _("The selected user group does not contain any members. Select another group or create a new user group.");
                                if ("_agents" in selectedQueue) {
                                    if (selectedQueue._agents.length < 1) {
                                        isValid = false;
                                        Tooltip.show(noAgentsMsg, registry.byId(miWindow.id + "_groupId").domNode);
                                    }
                                } else {
                                    new Store({ target: "/aps/2/resources/" + currentModel.agentQueueId + "/agents/" }).query().then(function(agents) {
                                        selectedQueue["_agents"] = agents;
                                        if (agents.length < 1) {
                                            isValid = false;
                                            Tooltip.show(noAgentsMsg, registry.byId(miWindow.id + "_groupId").domNode);
                                        }

                                        validationResult.resolve(isValid);
                                    }, function() {
                                        Tooltip.show(_("Cannot validate the selected user group. Check whether this user group was not removed."), registry.byId(miWindow.id + "_groupId").domNode);
                                        validationResult.resolve(false);
                                    });
                                    return validationResult;
                                }
                            }
                        } else {
                            isValid = isValid && registry.byId(miWindow.id + "_message").validateMessage();
                        }

                        validationResult.resolve(isValid);
                        return validationResult;
                    };

                    var onSubmitResult = new Deferred();
                    validateMenuItem().then(function(isValid) {
                        if (!isValid) {
                            onSubmitResult.resolve(false);
                            return;
                        }

                        query(".newVoiceMenuWarning", self.domNode).style("display", "none");
                        query(".active", self.domNode).removeClass("active");

                        if (!miWindow._processedItem) {
                            // create a new menu item
                            var parent = registry.byId(miWindow._parentMenuItemId);
                            var response = getStateful({
                                "voiceResponses": null,
                                "keypadResponse": currentModel.selectedKey,
                                "action": currentModel.action,
                                "_isValid": currentModel.action == window.constants.toQueueTxt,
                                "_errorCode": currentModel.action == window.constants.toQueueTxt ? "" : "noResponses",
                                "question": {
                                    "question": {
                                        "messageType": window.constants.textToSpeechTxt,
                                        "textToSpeech": "",
                                        "audioFile": {
                                            name: null,
                                            identity: null,
                                            _selectedFile: null
                                        }
                                    },
                                    "responses": []
                                },
                                "agentQueueId": "new",
                                "_groupName": "",
                                "_groupUsers": []
                            });

                            if (currentModel.action == window.constants.toQueueTxt) {
                                response.set("agentQueueId", currentModel.agentQueueId);
                                response.set("_groupName", currentModel._groupName);
                                if (currentModel.agentQueueId == "new") {
                                    response.set("_groupUsers", lang.clone(currentModel._groupUsers));
                                }
                            } else {
                                if (currentModel.question.messageType == window.constants.textToSpeechTxt) {
                                    response.question.question.set("textToSpeech", currentModel.question.textToSpeech);
                                } else {
                                    response.question.question.audioFile.set("name", currentModel.question.audioFile.name);
                                    response.question.question.audioFile.identity = currentModel.question.audioFile.identity;
                                    response.question.question.audioFile._selectedFile = currentModel.question.audioFile._selectedFile;
                                    response.question.question.audioFile.set("_agreementConfirmed", true);
                                }
                                response.question.question.set("messageType", currentModel.question.messageType);
                            }

                            parent.item.question.responses.push(response);
                            if (!parent.item._isValid && parent.item._errorCode == "noResponses" && parent.item.question.responses.length > 1) {
                                parent.item.set("_isValid", true);
                                parent.item.set("_errorCode", "");
                            }

                            var child = new VoiceMenuItem({
                                item: response,
                                parent: parent.item,
                                parentMenuItemId: miWindow._parentMenuItemId,
                                level: parent.level + 1,
                                onItemClick: miWindow._onMenuItemClick,
                                onRemove: miWindow._onMenuItemRemove
                            });

                            child.addChild(new VoiceMenuItem({
                                level: child.level + 1,
                                iconClass: 'voiceSet',
                                phoneKey: '+',
                                id: child.id + "_setNew",
                                parent: child.item,
                                parentMenuItemId: child.id,
                                onItemClick: miWindow._onNewMenuItemClick
                            }));

                            parent.addChild(child);
                            parent.layout();

                            if (parent.item.question.responses.length == 9) {
                                registry.byId(parent.id + "_setNew").set("visible", false);
                            }
                        } else {

                            miWindow._processedItem.set("keypadResponse", currentModel.selectedKey);
                            miWindow._processedItem.set("action", currentModel.action);
                            if (currentModel.action == window.constants.toQueueTxt) {
                                miWindow._processedItem.set("agentQueueId", currentModel.agentQueueId);
                                miWindow._processedItem.set("_groupName", currentModel._groupName);
                                if (currentModel.agentQueueId == "new") {
                                    miWindow._processedItem._groupUsers = lang.clone(currentModel._groupUsers);
                                }
                            } else {
                                miWindow._processedItem.set("agentQueueId", null);
                                miWindow._processedItem.set("_groupName", null);
                                miWindow._processedItem.set("_groupUsers", []);
                            }

                            if (currentModel.action == window.constants.toQuestionTxt || currentModel.level === 0) {
                                if (currentModel.question.messageType == window.constants.textToSpeechTxt) {
                                    miWindow._processedItem.question.question.set("textToSpeech", currentModel.question.textToSpeech);
                                    miWindow._processedItem.question.question.audioFile.set("name", "");
                                    miWindow._processedItem.question.question.audioFile.identity = "";
                                    miWindow._processedItem.question.question.audioFile._selectedFile = null;
                                    miWindow._processedItem.question.question.audioFile.set("_agreementConfirmed", false);
                                } else {
                                    miWindow._processedItem.question.question.set("textToSpeech", "");
                                    miWindow._processedItem.question.question.audioFile.set("name", currentModel.question.audioFile.name);
                                    miWindow._processedItem.question.question.audioFile.identity = currentModel.question.audioFile.identity;
                                    miWindow._processedItem.question.question.audioFile._selectedFile = currentModel.question.audioFile._selectedFile;
                                    miWindow._processedItem.question.question.audioFile.set("_agreementConfirmed", true);
                                }
                                miWindow._processedItem.question.question.set("messageType", currentModel.question.messageType);
                            } else {
                                miWindow._processedItem.question.question.set("textToSpeech", "");
                                miWindow._processedItem.question.question.audioFile.set("name", "");
                                miWindow._processedItem.question.question.audioFile.identity = "";
                                miWindow._processedItem.question.question.audioFile._selectedFile = null;
                                miWindow._processedItem.question.question.audioFile.set("_agreementConfirmed", false);
                                miWindow._processedItem.question.question.set("messageType", window.constants.textToSpeechTxt);
                            }

                            miWindow._processedItem.set("_isValid", (currentModel.action == window.constants.toQueueTxt || miWindow._processedItem.question.responses.length > 1));
                            miWindow._processedItem.set("_errorCode", currentModel.action == window.constants.toQueueTxt ? "" : "noResponses");
                        }

                        miWindow._processedItem = null;
                        miWindow._parentMenuItemId = null;
                        self.domNode.style["minHeight"] = "";
                        self._updateUserGroupCount();
                        onSubmitResult.resolve(true);
                    });

                    return onSubmitResult;
                }
            });

            var keypadContainer = new Container({
                title: _("Phone Key"),
                visible: at(self._menuItemWindow.model, "level").direction(at.from).transform({
                    format: function(value) {
                        return value > 0;
                    }
                })
            });
            keypadContainer.addChild(new KeypadSelector({
                selectedKey: at(self._menuItemWindow.model, "selectedKey"),
                disabledKeys: at(self._menuItemWindow.model, "disabledKeys").direction(at.from)
            }));
            self._menuItemWindow.addChild(keypadContainer);

            var actionContainer = new Container({
                title: _("Action"),
                visible: at(self._menuItemWindow.model, "level").direction(at.from).transform({
                    format: function(value) {
                        return value < 2;
                    }
                })
            });
            actionContainer.addChild(new RadioButton({
                description: _("Forward call to user group"),
                name: self._menuItemWindow.id + "_action",
                checked: at(self._menuItemWindow.model, 'action').transform({
                    format: self._isQueue,
                    parse: function(value) { return value ? window.constants.toQueueTxt : window.constants.toQuestionTxt; }
                })
            }));
            actionContainer.addChild(new RadioButton({
                description: at(self._menuItemWindow.model, "level").direction(at.from).transform({
                    format: function(value) {
                        switch (value) {
                        case 0:
                            return _("Play message and start voice menu");
                        default:
                            return _("Play message and start 2nd-level voice menu");
                        }
                    }
                }),
                name: self._menuItemWindow.id + "_action",
                checked: at(self._menuItemWindow.model, 'action').transform({
                    format: self._isQuestion,
                    parse: function(value) { return value ? window.constants.toQuestionTxt : window.constants.toQueueTxt; }
                })
            }));
            self._menuItemWindow.addChild(actionContainer);

            var questionContainer = new Container({
                title: _("Message"),
                visible: at(self._menuItemWindow.model, 'action').direction(at.from).transform({
                    format: function(value) {
                        return (value == window.constants.toQuestionTxt || this.source.level === 0);
                    }
                })
            });

            self._menuItemWindow.model.watch("level", function (propName, oldValue, newValue) {
                if (propName === "level" && oldValue != newValue) {
                    questionContainer.set("visible", newValue === 0 || self._menuItemWindow.model.action == window.constants.toQuestionTxt);
                }
            });

            questionContainer.addChild(new VoiceMenuMessage({
                message: self._menuItemWindow.model.question,
                id: self._menuItemWindow.id + "_message"
            }));
            self._menuItemWindow.addChild(questionContainer);

            var userGroupContainer = new Container({
                title: _("Forward call to user group"),
                visible: at(self._menuItemWindow.model, 'action').direction(at.from).transform({ format: self._isQueue })
            });

            var userGroupSelector = new Select({
                store: self.existingQueues,
                value: at(self._menuItemWindow.model, 'agentQueueId'),
                labelAttr: 'name',
                id: self._menuItemWindow.id + "_groupId"
            });

            self._menuItemWindow.own(
                on(userGroupSelector, mouse.leave, lang.hitch(userGroupSelector, function () {
                    Tooltip.hide(this.domNode);
                })),
                on(userGroupSelector, "click", lang.hitch(userGroupSelector, function () {
                    Tooltip.hide(this.domNode);
                }))
            );
            userGroupContainer.addChild(userGroupSelector);

            userGroupContainer.addChild(new TextBox({
                value: at(self._menuItemWindow.model, '_groupName'),
                id: self._menuItemWindow.id + "_groupName",
                placeHolder: _("Group title, f.e. __example__", { example: "Sales" }),
                visible: at(self._menuItemWindow.model, 'agentQueueId').direction(at.from).transform({ format: self._isNewGroup }),
                disabled: at(self._menuItemWindow.model, '_isNewGroupAllowed').direction(at.from).transform({ format: self._isNot }),
                onChange: utils.trimOnChange,
                validator: function (value) {
                    if (!self._menuItemWindow.visible ||
                        self._menuItemWindow.model.action !== window.constants.toQueueTxt ||
                        !self._isNewGroup(self._menuItemWindow.model.agentQueueId)) {
                        return true;
                    }

                    value = utils.trimStr(value);
                    if (!value) {
                        this.invalidMessage = _("User group title is not specified.");
                        return false;
                    }

                    var existing = self.existingQueues.query({ name: value });
                    if (existing.length > 0) {
                        this.invalidMessage = _("User group with this title already exists. Please, specify other title.");
                        return false;
                    }

                    return true;
                }
            }));

            userGroupContainer.addChild(new Output({
                id: self._menuItemWindow.id + "_noUserGroupError",
                value: "<span class='voiceMenuItemErrorMsg'>" + _("It is not possible to create a new user group because the limit is reached. Select an existing user group.") + "</span>",
                escapeHTML: false,
                visible: at(self._menuItemWindow.model, 'agentQueueId').direction(at.from).transform({ format: function(value) {
                    return self._isNewGroup(value) && !self._menuItemWindow.model._isNewGroupAllowed;
                } })
            }));

            self.own(
                self._menuItemWindow.model.watch("_isNewGroupAllowed", function (propName, oldValue, newValue) {
                    if (propName === "_isNewGroupAllowed" && oldValue != newValue) {
                        registry.byId(self._menuItemWindow.id + "_noUserGroupError").set("visible", !newValue && self._isNewGroup(this.agentQueueId));
                    }
                })
            );

            var msGroupUsers = new MultiSelect({
                store: self.lyncUsers,
                titleLeft: _('All Available Users'),
                titleRight: _('Group Members'),
                labelAttr: 'displayName',
                visible: at(self._menuItemWindow.model, 'agentQueueId').direction(at.from).transform({ format: self._isNewGroup }),
                values: self._menuItemWindow.model._groupUsers,
                id: self._menuItemWindow.id + "_groupUsers",
                disabled: at(self._menuItemWindow.model, '_isNewGroupAllowed').direction(at.from).transform({ format: self._isNot })
            });

            self._menuItemWindow.own(
                on(msGroupUsers, mouse.leave, lang.hitch(msGroupUsers, function() {
                    Tooltip.hide(this.domNode);
                })),
                on(msGroupUsers, "click", lang.hitch(msGroupUsers, function () {
                    Tooltip.hide(this.domNode);
                }))
            );

            userGroupContainer.addChild(msGroupUsers);
            self._menuItemWindow.addChild(userGroupContainer);
            self.addChild(self._menuItemWindow);
        },

        _getDisabledKeys: function(response, selectedKey) {
            if (!response || response.action == window.constants.toQueueTxt ||
                !response.question || !response.question.responses ||
                response.question.responses.length === 0) {
                return [];
            }

            return response.question.responses.map(function(item) {
                return item.keypadResponse;
            }).filter(function(item) { return item != selectedKey; });
        },

        getNewGroups: function() {
            // summary:
            //  Returns name and members of new groups defined in the menu.

            var newGroups = [],
                self = this;

            function getNewGroup(menuItem) {
                if (!menuItem._isValid) {
                    return;
                }

                if (menuItem.action == window.constants.toQueueTxt) {
                    if (menuItem.agentQueueId === "new") {
                        newGroups.push({
                            newGroupName: menuItem._groupName,
                            groupUsers: menuItem._groupUsers
                        });
                    }
                } else {
                    menuItem.question.responses.forEach(getNewGroup);
                }
            }

            getNewGroup(self.menu);
            return newGroups;
        },

        uploadAudioFiles: function (url, rgsIdentity) {
            // summary:
            //      upload a new audio files selected for voice menu items.
            // returns:
            //      dojo/Deferred
            var deferredFileUpload = new Deferred();
            var self = this,
                filesToUpload = [];

            function getItemFiles(menuItem, isTopLevel) {
                if (menuItem._isValid && (isTopLevel || menuItem.action == window.constants.toQuestionTxt)) {
                    if (menuItem.question.question.messageType == window.constants.audioFileTxt && !menuItem.question.question.audioFile.identity) {
                        filesToUpload.push(menuItem.question.question.audioFile);
                    }

                    menuItem.question.responses.forEach(getItemFiles);
                }
            }

            getItemFiles(self.menu, true);

            console.log("files to upload:");
            console.log(filesToUpload);
            if (filesToUpload.length > 0) {
                var reader = new FileReader();

                var readAndPost = function (files) {
                    if (files.length === 0) {
                        var page = registry.byId("pageContainer"), messages = page.get("messageList");
                        messages.addChild(new Message({ description: _("Files successfully uploaded!"), type: "info" }));
                        deferredFileUpload.resolve();
                    }

                    var file = files.splice(0, 1)[0];

                    var onFileLoaded = function (evt) {
                        if (evt.target.readyState == FileReader.DONE) {
                            var base64Content = utils.parseFileReaderResult(evt.target.result);
                            when(xhr.post(url, {
                                data: base64Content,
                                query: {
                                    rgServiceIdentity: rgsIdentity,
                                    name: file._selectedFile.name
                                }
                            }), function (data) {
                                    file.set("identity", data);
                                    readAndPost(files);
                            }, function (err) {
                                    deferredFileUpload.reject(err);
                            });
                        }
                    };

                    reader.onloadend = onFileLoaded;
                    if (reader.readAsBinaryString) {
                        reader.readAsBinaryString(file._selectedFile);
                    } else {
                        // process IE 10+
                        reader.readAsDataURL(file._selectedFile);
                    }
                };

                readAndPost(filesToUpload);
            } else {
                deferredFileUpload.resolve();
            }

            return deferredFileUpload;
        },

        updateUserGroups: function (createdGroups) {
            // summary:
            //      updates the user groups ids of voice menu items
            // createdGroups:
            //      maps names of created groups to their ids. (createdGroups[groupName] = group.aps.id)

            var self = this;
            function updateMenuItem(menuItem) {
                if (menuItem.action == window.constants.toQueueTxt) {
                    if (menuItem.agentQueueId == "new" && menuItem._groupName in createdGroups) {
                        menuItem.set("agentQueueId", createdGroups[menuItem._groupName]);
                    }
                } else {
                    if (menuItem.question.responses && menuItem.question.responses.length > 0) {
                        menuItem.question.responses.forEach(updateMenuItem);
                    }
                }
            }

            updateMenuItem(self.menu);
        },

        getPlainMenu: function () {
            // summary:
            //      creates and return call group menu basing on the current state
            var self = this;

            function removeExtraProperties(menuItem) {
                Object.keys(menuItem).forEach(function(key) {
                    if (key.indexOf("_") === 0) {
                        delete menuItem[key];
                    }
                });

                if (menuItem.action == window.constants.toQueueTxt) {
                    menuItem.question.responses = null;
                }

                if (menuItem.question.question.messageType == window.constants.textToSpeechTxt) {
                    menuItem.question.question.audioFile = null;
                } else {
                    delete menuItem.question.question.audioFile["_selectedFile"];
                    delete menuItem.question.question.audioFile["_agreementConfirmed"];
                }

                if (menuItem.action == window.constants.toQuestionTxt) {
                    menuItem.question.responses.forEach(removeExtraProperties);
                }
            }

            var plainMenu = getPlainValue(self.menu);
            removeExtraProperties(plainMenu);
            return plainMenu;
        },

        getAllRelatedGroups: function(getRow) {
            // summary:
            //      get all realted agent groups in form of [{aps: { id: apsId, "backrel": "voiceGreetings" }}..] or [apsId...]
            // getRow: boolean
            //      indicates that row ids should be returned

            var self = this,
                groups = [];

            function getRelatedGroup(menuItem) {
                if (menuItem.action == window.constants.toQueueTxt) {
                    if (groups.indexOf(menuItem.agentQueueId) < 0) {
                        groups.push(menuItem.agentQueueId);
                    }
                } else {
                    menuItem.question.responses.forEach(getRelatedGroup);
                }
            }

            getRelatedGroup(self.menu);
            if (getRow) {
                return groups;
            } else {
                return groups.map(function(item) {
                    return {
                        aps: { id: item, "backrel": "voiceGreetings" }
                    };
                });
            }
        },

        validateMenu: function() {
            var self = this,
                isValid = true;

            function validateMenuItem(menuItem) {
                if (!menuItem._isValid) {
                    isValid = false;
                }

                if (menuItem.action == window.constants.toQuestionTxt) {
                    menuItem.question.responses.forEach(validateMenuItem);
                }
            }

            validateMenuItem(self.menu);
            if (!isValid) {
                query(".voiceMenuItemError", self.domNode).style("display", "block");
            }

            return isValid;
        },

        _updateUserGroupCount: function() {
            // summary:
            //      re-counts the new user groups and updates _newUserGroupCount property

            var self = this,
                userGroupCount = 0;
            var countNewUserGroups = function(menuItem) {
                if (menuItem.action == window.constants.toQueueTxt) {
                    if (menuItem._isValid && menuItem.agentQueueId === "new") {
                        userGroupCount++;
                    }
                } else {
                    if (menuItem.question.responses && menuItem.question.responses.length > 0) {
                        menuItem.question.responses.forEach(countNewUserGroups);
                    }
                }
            };

            countNewUserGroups(self.menu);
            self._newUserGroupCount = userGroupCount;
        },

        postCreate: function () {
            var self = this;
            self.inherited(arguments);
            self._updateUserGroupCount();

            if (self.isNew) {
                var msgNew;
                self.own(
                    msgNew = new Message({
                        description: _("This is a sample configuration for the new call group. Modify this configuration to match your requirements."),
                        type: "warning",
                        closeable: false
                    })
                );
                domConstruct.place(msgNew.domNode, query(".newVoiceMenuWarning", self.domNode)[0]);
            }

            if (!window.constants.textToSpeechTxt)
                window.constants.textToSpeechTxt = "TextToSpeech";
            if (!window.constants.audioFileTxt)
                window.constants.audioFileTxt = "AudioFile";
            if (!window.constants.toQueueTxt)
                window.constants.toQueueTxt = "TransferToQueue";
            if (!window.constants.toQuestionTxt)
                window.constants.toQuestionTxt = "TransferToQuestion";

            if (self.phoneNumber) {
                self.own(
                    self.phoneNumber.watch("displayNumber", function (propName, oldValue, newValue) {
                        if (propName === "displayNumber" && oldValue != newValue) {
                            query(".voiceMenuDisplayPhone", this.domNode)[0].innerHTML = newValue;
                        }
                    })
                );
            }

            self._definePhoneNumberWindow();
            self._defineMenuItemWindow();

            if (!self.menu) {
                self.menu = getStateful({
                    "question": {
                        "question": {
                            "messageType": window.constants.textToSpeechTxt,
                            "textToSpeech": "Thank you for calling. We appreciate your call and will be with you shortly.",
                            "audioFile": {
                                name: null,
                                identity: null,
                                _selectedFile: null,
                                _agreementConfirmed: false
                            }
                        },
                        responses: [
                        ]
                    },
                    "action": window.constants.toQueueTxt,
                    "agentQueueId": "new",
                    "_groupName": "Sales",
                    "_groupUsers": [],
                    "_isValid": false
                });
            }

            var onMenuItemRemove = function () {
                this.parent.question.responses.splice(this.parent.question.responses.indexOf(this.item), 1);
                var parent = registry.byId(this.parentMenuItemId);
                parent.removeChild(this);
                if (this.parent.question.responses.length < 2) {
                    this.parent.set("_isValid", false);
                    this.parent.set("_errorCode", "noResponses");
                }

                if (this.parent.question.responses.length < 9) {
                    registry.byId(parent.id + "_setNew").set("visible", true);
                }
                query(".newVoiceMenuWarning", self.domNode).style("display", "none");

                self._updateUserGroupCount();
            };

            var openMenuItemWindow = function() {
                var item = this;
                item.markAsActive();
                self._menuItemWindow._processedItem = item.item;
                var plainItem = getPlainValue(item.item);
                if (plainItem._isValid && plainItem.agentQueueId === "new") {
                    self._newUserGroupCount--;
                }

                self._menuItemWindow.model.set("_isNewGroupAllowed", self.userGroupLimit === "unlimited" || self.userGroupLimit > self._newUserGroupCount);
                self._menuItemWindow.model.set("level", item.level);
                self._menuItemWindow.model.set("selectedKey", plainItem.keypadResponse);
                self._menuItemWindow.model.set("disabledKeys", self._getDisabledKeys(item.parent, plainItem.keypadResponse));
                self._menuItemWindow.model.set("action", plainItem.action);
                self._menuItemWindow.model.set("agentQueueId", plainItem.agentQueueId ? plainItem.agentQueueId : "new");
                self._menuItemWindow.model.set("_groupName", plainItem._groupName);

                self._menuItemWindow.model._groupUsers.set("length", 0);
                if (item.item._groupUsers.length > 0) {
                    item.item._groupUsers.forEach(function (user) { self._menuItemWindow.model._groupUsers.push(user); });
                }

                if (plainItem.action == window.constants.toQuestionTxt || item.level === 0) {
                    self._menuItemWindow.model.question.set("textToSpeech", plainItem.question.question.textToSpeech);
                    if (plainItem.question.question.audioFile) {
                        self._menuItemWindow.model.question.audioFile.set("name", plainItem.question.question.audioFile.name);
                        self._menuItemWindow.model.question.audioFile.identity = plainItem.question.question.audioFile.identity;
                        self._menuItemWindow.model.question.audioFile._selectedFile = plainItem.question.question.audioFile._selectedFile;
                        self._menuItemWindow.model.question.audioFile.set("_agreementConfirmed", true);
                    } else {
                        self._menuItemWindow.model.question.audioFile.set("name", "");
                        self._menuItemWindow.model.question.audioFile.identity = "";
                        self._menuItemWindow.model.question.audioFile._selectedFile = null;
                        self._menuItemWindow.model.question.audioFile.set("_agreementConfirmed", false);
                    }
                    self._menuItemWindow.model.question.set("messageType", plainItem.question.question.messageType);
                } else {
                    self._menuItemWindow.model.question.set("textToSpeech", "");
                    self._menuItemWindow.model.question.audioFile.set("name", "");
                    self._menuItemWindow.model.question.audioFile.identity = "";
                    self._menuItemWindow.model.question.audioFile._selectedFile = null;
                    self._menuItemWindow.model.question.audioFile.set("_agreementConfirmed", false);
                    self._menuItemWindow.model.question.set("messageType", window.constants.textToSpeechTxt);
                }

                self._menuItemWindow.show();
                // positioning
                var itemPosition = domGeometry.position(query(".voiceMenuItemTitle", item.domNode)[0]);
                self._menuItemWindow.domNode.style["top"] = itemPosition.y + "px";
                self._menuItemWindow.domNode.style["left"] = item.level === 0 ? "40px" : (itemPosition.x + "px");

                var voiceMenuPosition = domGeometry.position(self.domNode);
                var position = domGeometry.position(self._menuItemWindow.domNode);
                var maxHeight = self._menuItemWindow.maxHeight ? self._menuItemWindow.maxHeight : position.h;
                if (position.y + maxHeight > voiceMenuPosition.h) {
                    self.domNode.style["minHeight"] = (position.y - voiceMenuPosition.y + maxHeight) + "px";
                }
            };

            var openWindowForNewItem = function() {
                var item = this;
                item.markAsActive();
                var disabledKeys = self._getDisabledKeys(item.parent, null);
                if (disabledKeys.length == 9) {
                    console.log("voice set should be already hidden!");
                    alert("No more actions can be added at this voice menu level: all keys are ocupied.");
                    item.domNode.style.display = "none";
                    return;
                }

                var selectedKey, keys = ["1", "2", "3", "4", "5", "6", "7", "8", "9"];
                for (var index = 0; index < 9; index++) {
                    selectedKey = keys[index];
                    if (disabledKeys.indexOf(selectedKey) < 0) {
                        break;
                    }
                }

                self._menuItemWindow._processedItem = null;
                self._menuItemWindow._parentMenuItemId = item.parentMenuItemId;
                self._menuItemWindow._onMenuItemClick = openMenuItemWindow;
                self._menuItemWindow._onNewMenuItemClick = openWindowForNewItem;
                self._menuItemWindow._onMenuItemRemove = onMenuItemRemove;
                self._menuItemWindow.model.set("level", item.level);
                self._menuItemWindow.model.set("selectedKey", selectedKey);
                self._menuItemWindow.model.set("disabledKeys", disabledKeys);

                self._menuItemWindow.model.set("_isNewGroupAllowed", self.userGroupLimit === "unlimited" || self.userGroupLimit > self._newUserGroupCount);

                self._menuItemWindow.model.set("action", window.constants.toQueueTxt);
                self._menuItemWindow.model.set("agentQueueId", "new");
                self._menuItemWindow.model.set("_groupName", "");
                self._menuItemWindow.model._groupUsers.set("length", 0);

                self._menuItemWindow.model.question.set("textToSpeech", "");
                self._menuItemWindow.model.question.audioFile.set("name", "");
                self._menuItemWindow.model.question.audioFile.identity = "";
                self._menuItemWindow.model.question.audioFile._selectedFile = null;
                self._menuItemWindow.model.question.audioFile.set("_agreementConfirmed", false);
                self._menuItemWindow.model.question.set("messageType", window.constants.textToSpeechTxt);

                self._menuItemWindow.show();
                // positioning
                var itemPosition = domGeometry.position(query(".voiceMenuItemTitle", item.domNode)[0]);
                self._menuItemWindow.domNode.style["top"] = itemPosition.y + "px";
                self._menuItemWindow.domNode.style["left"] = item.level === 0 ? "40px" : (itemPosition.x + "px");

                var voiceMenuPosition = domGeometry.position(self.domNode);
                var position = domGeometry.position(self._menuItemWindow.domNode);
                var maxHeight = self._menuItemWindow.maxHeight ? self._menuItemWindow.maxHeight : position.h;
                if (position.y + maxHeight > voiceMenuPosition.h) {
                    self.domNode.style["minHeight"] = (position.y - voiceMenuPosition.y + maxHeight) + "px";
                }
            };

            self._menuItem = new VoiceMenuItem({
                item: self.menu,
                level: 0,
                id: self.id + "_menu",
                parent: null,
                onItemClick: openMenuItemWindow
            });

            self.addChild(self._menuItem);

            if (self.menu.action === window.constants.toQuestionTxt) {
                var processResponse = function(response) {
                    if (response.action == window.constants.toQueueTxt && !response._groupName && !!response.agentQueueId && response.agentQueueId != "new" && self.existingQueues != null) {
                        var group = self.existingQueues.get(response.agentQueueId);
                        if (!!group) {
                            response.set("_groupName", group.name);
                        }
                    }

                    var parent = this;
                    var child = new VoiceMenuItem({
                        item: response,
                        parent: parent.item,
                        parentMenuItemId: parent.id,
                        level: parent.level + 1,
                        onItemClick: openMenuItemWindow,
                        onRemove: onMenuItemRemove
                    });

                    parent.addChild(child);
                    if (response.action === window.constants.toQuestionTxt) {
                        if (response.question.responses) {
                            response.question.responses.forEach(processResponse, child);
                        }
                    }

                    child.addChild(new VoiceMenuItem({
                        level: child.level + 1,
                        iconClass: 'voiceSet',
                        phoneKey: '+',
                        id: child.id + "_setNew",
                        parent: child.item,
                        parentMenuItemId: child.id,
                        onItemClick: openWindowForNewItem
                    }));
                };

                if (self.menu.question.responses) {
                    self.menu.question.responses.forEach(processResponse, self._menuItem);
                }
            }

            self._menuItem.addChild(new VoiceMenuItem({
                level: 1,
                iconClass: 'voiceSet',
                phoneKey: '+',
                id: self._menuItem.id + "_setNew",
                parent: self._menuItem.item,
                parentMenuItemId: self._menuItem.id,
                onItemClick: openWindowForNewItem
            }));

            domConstruct.place(self._menuItem.domNode, self.domNode);
        }
    });
});