define([
    "dojo/_base/declare",
    "aps/_Widget",
    "aps/_Container",
    "aps/i18n",
    "aps/Output",
    "dijit/_TemplatedMixin",
    "dojo/dom-construct",
    "dojo/_base/array",
    "dojox/mvc/getStateful",
    "dojox/mvc/at",
    "dojo/query",
    "dojox/html/entities"
], function (
    declare,
    _Widget,
    _Container,
    _,
    Output,
    _Templated,
    domConstruct,
    arr,
    getStateful,
    at,
    query,
    entities) {
    //"use strict";
    return declare([_Widget, _Container, _Templated], {
        // summary:
        // 		Widget represents a voice menu item with title, phoneKey(optional) and icon.
        // description:
        // 		This widget should be used ONLY in VoiceGreetingMenu.


        // baseClass: [private] string
        // 		Root CSS class of the widget (ex: dijitTextBox),
        // 		used to construct CSS classes to indicate widget state.
        baseClass: "lyncVoiceMenuItem",

        // phoneKey: string
        //      A phone key associated with this voice menu item.
        phoneKey: "",

        // iconClass: string
        //      Defines item icon.
        //      Supported classes: voicePlay (default), voiceSupport, voiceMenu and voiceSet
        iconClass: "voicePlay",

        // content: string
        // 		Title of voice menu item.
        title: "",

        // level: int
        //      A level in the voice menu where this item is located.
        //      Default: 1
        level: 1,

        // showNestedItems: boolean
        //      Indicates whether the nexted voice menu items should be shown.
        //      Default: false
        showNestedItems: false,

        // item: stateful object representing the voice menu item model.
        item: null,

        // templateString: [private] string
        // 		Template string
        templateString: '<li><a class="${iconClass}" data-dojo-attach-event="ondijitclick:_onItemClick"><b class="phoneKey">${phoneKey}</b><span class="arrowRight"></span><p class="voiceMenuItemTitle">${title}</p></a><a class="voiceRemove" data-dojo-attach-event="ondijitclick:_onItemRemove"></a><span class="voiceMenuItemError"></span><div class="voiceMenuItemList" style="display: none"><ul></ul></div></li>',

        // validateOnRemove: function returning boolean
        //      a custom function allowing to validate whether item can be removed.
        validateOnRemove: null,

        // onRemove: function (item)
        //      a custom function allowing to perform additional action when this voice menu item is removed.
        onRemove: null,

        // onItemClick: function
        //      a custom function allowing to perform additional action when this voice menu item is clicked.
        onItemClick: null,

        markAsActive: function () {
            // summary:
            //      Marks this menu item as active
            // tags:
            // 		public

            var el = this.level === 0 ? query("a", this.domNode)[0] : query(".phoneKey", this.domNode)[0];
            if (el.classList) {
                el.classList.add("active");
            } else {
                el.className = "active";
            }
        },

        _onItemRemove: function() {
            // summary:
            //      processes a click on remove button
            // tags:
            // 		private

            if (!this.validateOnRemove || this.validateOnRemove()) {
                if (this.onRemove) this.onRemove(this);
                this.destroy();
            }
        },

        _onItemClick: function () {
            // summar:
            //      processes a cick on voice menu item.
            // tags:
            // 		private

            if (this.onItemClick) {
                this.onItemClick();
            }
        },

        // _voiceSetNode:
        //      node of voiceSet menu item (allows to create a new menu item)
        _voiceSetNode: null,

        // item's text strings.
        _forwardToGroupText: function(groupName) {
            return _("Forward to __userGroupName__", { 'userGroupName': groupName });
        },
        _forwardToTextMenuText: _("Play Text Message and Start Voice Menu"),
        _forwardToAudioMenuText: _("Play Audio File and Start Voice Menu"),
        _setKeyText: _("Set Key"),

        // _errors:
        //      defines the possible error messages.
        _errors: {
            "noAgents": _("Members are not defined for user group."),
            "noResponses": _("Voice menu should contain at least 2 items."),
            "default": _("Please, complete the creation of this menu item.")
        },

        // properties setters
        _setTitleAttr: function(val) {
            var encodedValue = entities.encode(val);
            query(".voiceMenuItemTitle", this.domNode)[0].innerHTML = encodedValue;
            this._set("title", encodedValue);
        },

        _setShowNestedItemsAttr: function(val) {
            query(".voiceMenuItemList", this.domNode)[0].style.display = val ? "" : "none";
            this._set("showNestedItems", val);
        },

        _setLevelAttr: function (val) {
            var isRoot = val === 0;
            query(".phoneKey", this.domNode)[0].style.display = isRoot ? "none" : "";
            query(".arrowRight", this.domNode)[0].style.display = isRoot ? "none" : "";
            query(".voiceRemove", this.domNode)[0].style.display = isRoot || this.iconClass == "voiceSet" ? "none" : "";
            this._set("level", val);
        },

        _setPhoneKeyAttr: function(val) {
            if (this.level !== 0) {
                query(".phoneKey", this.domNode)[0].innerHTML = val;
            }

            this._set("phoneKey", val);
        },
        _setIconClassAttr: function(val) {
            if (this.iconClass) {
                var item = query("." + this.iconClass, this.domNode)[0];
                item.className = val;
            }

            query(".voiceRemove", this.domNode)[0].style.display = val == "voiceSet" || this.level === 0 ? "none" : "";
            if (val == "voiceSet") {
                this.set("title", this._setKeyText);
            }
            this._set("iconClass", val);
        },

        layout: function() {
            // summary:
            //		Widgets override this method to size and position their contents/children.
            //		This is called after startup().
            // tags:
            // 		Public

            var self = this,
                ul = query("ul", self.domNode)[0],
                messageList = query(".voiceMenuItemError", self.domNode)[0];

            arr.forEach(this.getChildren(), function (child) {
                if (child.baseClass == "apsOutput") {
                    domConstruct.place(child.domNode, messageList);
                } else {
                    if (child.iconClass && child.iconClass == 'voiceSet') {
                        if (child.domNode.classList) {
                            child.domNode.classList.add("voiceLast");
                        } else {
                            child.domNode.className = "voiceLast";
                        }

                        domConstruct.place(child.domNode, ul);
                        self._voiceSetNode = child.domNode;
                    } else {
                        domConstruct.place(child.domNode, ul, self._voiceSetNode == null ? "last" : ul.children.length - 1);
                    }
                }
            });
        },

        postCreate: function () {
            // summary:
            //		Post create processing.
            // tags:
            // 		Public

            var self = this;
            self.inherited(arguments);

            if (self.item) {

                self.addChild(new Output({
                    content: at(self.item, "_errorCode").direction(at.from).transform({
                        format: function(value) {
                            return (!value || !(value in self._errors)) ? self._errors["default"] : self._errors[value];
                        }
                    }),
                    "class": "voiceMenuItemErrorMsg",
                    visible: at(self.item, "_isValid").direction(at.from).transform({
                        format: function (value) {
                            return (!value);
                        }
                    }),
                    closeable: false,
                    id: self.id + "_errorMrg"
                }));

                var isQueue = self.item.action === window.constants.toQueueTxt,
                    isText = !self.item.question || !self.item.question.question || self.item.question.question.messageType === window.constants.textToSpeechTxt;
                self.set('title', isQueue ? self._forwardToGroupText(self.item._groupName) : isText ? self._forwardToTextMenuText : self._forwardToAudioMenuText);
                self.set('iconClass', isQueue ? 'voiceSupport' : isText ? 'voiceMenu' : 'voicePlay');
                self.set('showNestedItems', !isQueue);
                self.set('phoneKey', self.item.keypadResponse);

                if (!self.item.question || !self.item.question.question) {
                    self.item.question = getStateful({
                        "question": {
                            "messageType": window.constants.textToSpeechTxt,
                            "textToSpeech": "",
                            "audioFile": {
                                name: null,
                                identity: null,
                                _selectedFile: null
                            }
                        },
                        responses: [
                        ]
                    });
                }

                if (!self.item.question.question.audioFile) {
                    self.item.question.question.audioFile = getStateful({
                        name: null,
                        identity: null,
                        _selectedFile: null
                    });
                }

                if (!self.item.question.responses) {
                    self.item.question.responses = [];
                }

                if (!self.item._groupName) {
                    self.item._groupName = "";
                }

                if (!self.item._groupUsers) {
                    self.item._groupUsers = [];
                }

                self.own(
                    self.item.watch("action", function(propName, oldValue, newValue) {
                        if (propName === "action" && oldValue != newValue) {
                            var isQueue = newValue === window.constants.toQueueTxt,
                                isText = !this.question || !this.question.question || this.question.question.messageType === window.constants.textToSpeechTxt;
                            self.set('title', isQueue ? self._forwardToGroupText(this._groupName) : isText ? self._forwardToTextMenuText : self._forwardToAudioMenuText);
                            self.set('iconClass', isQueue ? 'voiceSupport' : isText ? 'voiceMenu' : 'voicePlay');
                            self.set('showNestedItems', !isQueue);
                        }
                    }),
                    self.item.watch("_groupName", function (propName, oldValue, newValue) {
                        if (propName === "_groupName" && oldValue != newValue) {
                            var isQueue = self.item.action === window.constants.toQueueTxt,
                                isText = !this.question || !this.question.question || this.question.question.messageType === window.constants.textToSpeechTxt;
                            self.set('title', isQueue ? self._forwardToGroupText(newValue) : isText ? self._forwardToTextMenuText : self._forwardToAudioMenuText);
                        }
                    }),
                    self.item.question.question.watch("messageType", function (propName, oldValue, newValue) {
                        if (propName === "messageType" && oldValue != newValue) {
                            var isQueue = self.item.action === window.constants.toQueueTxt,
                                isText = newValue === window.constants.textToSpeechTxt;
                            self.set('title', isQueue ? self._forwardToGroupText(self.item._groupName) : isText ? self._forwardToTextMenuText : self._forwardToAudioMenuText);
                            self.set('iconClass', isQueue ? 'voiceSupport' : isText ? 'voiceMenu' : 'voicePlay');
                        }
                    }),
                    self.item.watch("keypadResponse", function(propName, oldValue, newValue) {
                        if (propName === "keypadResponse" && oldValue != newValue) {
                            self.set('phoneKey', newValue);
                        }
                    })
                );
            }
        }
    });
});