define([
    "dojo/_base/declare",
    "dojo/on",
    "aps/Container",
    "aps/i18n",
    "aps/Message",
    "aps/Output",
    "aps/RadioButton",
    "aps/ToolbarButton",
    "aps/Tooltip",
    "aps/_Widget",
    "aps/_Container",
    "dijit/_TemplatedMixin",
    "dijit/registry",
    "dojo/dom-construct",
    "dojo/_base/array",
    "dojo/mouse",
    "dojo/query",
    "dojox/mvc/at",
    "dojox/mvc/getStateful",
    "./getActualLyncIntegrationSettings.js",
    "./TextToSpeechArea.js",
    "./utils.js"
], function (
    declare,
    on,
    Container,
    _,
    Message,
    Output,
    RadioButton,
    ToolbarButton,
    Tooltip,
    _Widget,
    _Container,
    _Templated,
    registry,
    domConstruct,
    arr,
    mouse,
    query,
    at,
    getStateful,
    getActualLyncIntegrationSettings,
    TextToSpeechArea,
    utils
) {
    return declare([_Widget, _Container, _Templated], {
        // summary:
        // 		Widget represents a voice menu item with title, phoneKey(optional) and icon.
        // description:
        // 		This widget should be used ONLY in VoiceMenu.


        // baseClass: [private] string
        // 		Root CSS class of the widget (ex: dijitTextBox),
        // 		used to construct CSS classes to indicate widget state.
        baseClass: "voiceMenuMessage",

        // message: stateful object representing the voice menu message.
        message: null,

        // templateString: [private] string
        // 		Template string
        templateString: '<table><col width="135"><col><tr><td class="voiceMenuMessageSwitch"></td><td class="voiceMenuMessageMsg"><form id="fileForm" method="POST" enctype="multipart/form-data"><input id="audioFileSelector" type="file" accept="${_audioFormats}" style="display:none" /></form></td></tr><tr><td colspan="2" class="voiceMenuMessageWarn"></td></tr></table>',

        _audioFormats: utils.getAudioSupportedFormatsString(),

        _isText: function (value) { return value == window.constants.textToSpeechTxt; },
        _isAudio: function (value) { return value == window.constants.audioFileTxt; },
        _setAudio: function (value) { return value ? window.constants.audioFileTxt : window.constants.textToSpeechTxt; },
        _setTextToSpeech: function (value) { return value ? window.constants.textToSpeechTxt : window.constants.audioFileTxt; },

        _items: [],

        _actualLyncIntegrationSettingsRequest: getActualLyncIntegrationSettings(aps.context.vars.organization.aps.id),
        _agreementRequired: null,
        agreementMessage: null,

        _hideTooltips: function () {
            // summary:
            //      hides tooltips
            var self = this;
            Tooltip.hide(registry.byId(self.id + "_textToSpeech").domNode);
            Tooltip.hide(registry.byId(self.id + "_audioFile").domNode);
            var agreementArea = registry.byId(self.id + "_agreement");
            if (agreementArea) {
                Tooltip.hide(agreementArea.domNode);
            }
        },

        validateMessage: function () {
            // summary:
            //      validates the specified message
            var self = this;
            if (self.message.messageType == window.constants.textToSpeechTxt) {
                if (!self.message.textToSpeech) {
                    Tooltip.show(_("Text-to-speech is not specified."), registry.byId(self.id + "_textToSpeech").domNode, ["below"]);
                    return false;
                }
            } else {
                if (!self.message.audioFile || (!self.message.audioFile.name && !self.message.audioFile.identity)) {
                    Tooltip.show(_("The audio file is not specified."), registry.byId(self.id + "_audioFile").domNode, ["after"]);
                    return false;
                }
                if (self._agreementRequired && !self.message.audioFile._agreementConfirmed) {
                    Tooltip.show(_("Before you can proceed, please select the checkbox to confirm you agree with the terms."), registry.byId(self.id + "_agreement").domNode, ["before"]);
                    return false;
                }
            }

            return true;
        },

        postCreate: function () {
            // summary:
            //		Post create processing.
            // tags:
            // 		Public

            var self = this;
            self.inherited(arguments);
            var isFileSupported = utils.isFileApiSupported();

            var inputNode = query("input", this.domNode)[0];
            self.own(
                on(inputNode, "change", function () {
                    var files = this.files;
                    if (!utils.validateSelectedAudioFile(files))
                        return;

                    self.message.audioFile._selectedFile = files[0];
                    self.message.audioFile.set('name', files[0].name);
                    self.message.audioFile.set('identity', null);
                    self.message.audioFile.set('_agreementConfirmed', false);
                    document.getElementById("fileForm").reset();
                })
            );

            var switchArea = query(".voiceMenuMessageSwitch", this.domNode)[0];
            var rbText = new RadioButton({
                name: self.id + "_msg",
                description: _("Text-to-speech"),
                checked: at(self.message, 'messageType').transform({ format: self._isText, parse: self._setTextToSpeech })
            });

            domConstruct.place(rbText.domNode, switchArea);
            self._items.push(rbText);
            var rbAudio = new RadioButton({
                name: self.id + "_msg",
                description: _("Audio File"),
                checked: at(self.message, 'messageType').transform({ format: self._isAudio, parse: self._setAudio })
            });

            domConstruct.place(rbAudio.domNode, switchArea);
            self._items.push(rbAudio);

            var msgArea = query(".voiceMenuMessageMsg", this.domNode)[0];
            var textArea = new TextToSpeechArea({
                cols: 40,
                rows: 5,
                size: 4096,
                value: at(self.message, 'textToSpeech'),
                visible: at(self.message, 'messageType').direction(at.from).transform({ format: self._isText }),
                id: self.id + "_textToSpeech"
            });

            domConstruct.place(textArea.domNode, msgArea);
            self._items.push(textArea);

            var fileNameField = new Output({
                value: at(self.message.audioFile, 'name').direction(at.from).transform({
                    format: function (value) {
                        return (!value) ? _('No audio file specified yet.') : value;
                    }
                }),
                visible: at(self.message, 'messageType').direction(at.from).transform({ format: self._isAudio }),
                id: self.id + "_audioFile"
            });

            domConstruct.place(fileNameField.domNode, msgArea);
            self._items.push(fileNameField);

            var fileUploadBtn = new ToolbarButton({
                label: at(self.message.audioFile, 'name').direction(at.from).transform({
                    format: function (value) {
                        return (!value) ? _("Upload File") : _("Upload Another File");
                    }
                }),
                iconClass: "sb-search",
                disabled: !isFileSupported,
                parentId: self.id,
                onClick: function () {
                    inputNode.click();
                    this.cancel();
                    Tooltip.hide(registry.byId(this.parentId + "_audioFile").domNode);
                },
                visible: at(self.message, 'messageType').direction(at.from).transform({ format: self._isAudio })
            });

            domConstruct.place(fileUploadBtn.domNode, msgArea);
            self._items.push(fileUploadBtn);

            var warnArea = query(".voiceMenuMessageWarn", self.domNode)[0];
            if (!isFileSupported && utils.isInternetExplorer()) {
                var warn = new Message({
                    description: _("File uploading is not supported in this version of Internet Explorer. Use Internet Explorer 10 or above."),
                    type: "warning",
                    closeable: false,
                    visible: at(self.message, 'messageType').direction(at.from).transform({ format: self._isAudio })
                });

                domConstruct.place(warn.domNode, warnArea);
                self._items.push(warn);
            } else {
                self.domNode.applyAgreementState = function(val) {
                    self.agreementMessage.set("cb_changed", true);
                    self.message.audioFile.set("_agreementConfirmed", val);
                };
                self._actualLyncIntegrationSettingsRequest.then(function (lyncSettings) {
                    console.log(lyncSettings);
                    self._agreementRequired = utils.trimStr(lyncSettings.fileUploadAgreement) !== "";
                    var fileUploadAgreementString = utils.escapeHTML(lyncSettings.fileUploadAgreement).replace("\n", "<br/>\n");
                    self.agreementMessage = new Message({
                        description: "<input id='" + self.id + "_agreement_cb' type='checkbox' onclick='document.getElementById(\"" + self.id + "\").applyAgreementState(this.checked)'/> " + fileUploadAgreementString,
                        type: "warning",
                        closeable: false,
                        escapeHTML: false,
                        visible: self.agreementVisible(),
                        id: self.id + "_agreement"
                    });

                    self.own(
                        on(self.agreementMessage, "click", function (e) {
                            Tooltip.hide(registry.getEnclosingWidget(e.target).domNode);
                        })
                    );

                    self.own(
                        on(self.agreementMessage, mouse.leave, function (e) {
                            Tooltip.hide(registry.getEnclosingWidget(e.target).domNode);
                        })
                    );

                    domConstruct.place(self.agreementMessage.domNode, warnArea);
                    self._items.push(self.agreementMessage);

                    self.message.watch("messageType", function (propName) {
                        if (propName === "messageType") {
                            self.agreementMessage.set("visible", self.agreementVisible());
                        }
                    });

                    self.message.audioFile.watch("_agreementConfirmed", function (propName, oldValue, newValue) {
                        if (propName === "_agreementConfirmed") {
                            self.agreementMessage.set("visible", self.agreementVisible());
                            if (!newValue || oldValue == newValue) {
                                self.agreementMessage.set("cb_changed", false);
                                document.getElementById(self.id + "_agreement_cb").checked = false;
                            }
                        }
                    });
                });
            }

            self.message.watch("messageType", function (propName) {
                if (propName === "messageType") {
                    self._hideTooltips();
                }
            });
        },

        agreementVisible: function() {
            var self = this;
            return self._agreementRequired && self._isAudio(self.message.messageType) && (!self.message.audioFile._agreementConfirmed || (self.agreementMessage && self.agreementMessage.cb_changed));
        },

        destroy: function () {
            var self = this;
            if (self._items) {
                self._items.forEach(function (item) { item.destroy(); });
            }
        }
    });
});