define(["dijit/registry", "aps/ResourceStore", "aps/i18n"],
    function(registry, Store, _) {
        var extensionTools = {

            /// Function returns user licence who has 'val' extension assigned
            /// or null of none or same user as currentUserLicApsID
            isExtensionFree: function (val, existing, currentUserLicApsID) {
                for (var i = 0; i < existing.length; i++) {
                    if (existing[i].extension == val && (currentUserLicApsID == null || existing[i].aps.id != currentUserLicApsID)) {
                        return existing[i];
                    }
                }
                return null;
            },

            /// Function checks if there are users with assigned extension
            /// less than 'val' and returns number of found users.
            checkAssignedExtensionsLessThan: function (globals, existing, val) {
                if (existing == null) {
                    return 0;
                }
                var found = 0;
                for (var i = 0; i < existing.length; i++) {
                    if (existing[i].extension != null && existing[i].extension < val) {
                        found += 1;
                    }
                }
                return found;
            },

            /// Load users of organization with assigned extensions
            /// List is desc ordered by extension value
            loadExtensions: function (organization, successCallback, errorCallback) {
                var extensionsStore = new Store({
                    target: "/aps/2/resources/" + organization.aps.id + "/userLicenses?not(isnull(extension)),sort(-extension)",
                    apsType: "http://www.parallels.com/LyncHPv2/userLicense/1.5"
                });
                extensionsStore.query().then(successCallback, errorCallback);
            },

            /// Generate next acceptable extension value
            /// Returns null of failed to find any or a valid extension
            nextExtension: function (globals, organization, existing) {
                var last = organization.initialExtension;
                if (existing.length > 0 && existing[0].extension > last) {
                    last = existing[0].extension;
                }
                //check if last + 1 fits into any range, no ranges like [200-299],[300-399] are expected.
                //so last + 1 is either fits into range for last and next range does not start with last + 1.
                if (!this.isExtensionInRange(globals, last + 1)) {
                    //sort ranges by start value
                    globals.validExtensionRanges.sort(function (a, b) { return a.start - b.start; });
                    //find range which holds last assigned value
                    for (var i = 0; i < globals.validExtensionRanges.length - 1; i++) {
                        var range = globals.validExtensionRanges[i];
                        if (last >= range.start && last <= range.end) {
                            //return next range's start
                            return globals.validExtensionRanges[i + 1].start;
                        }
                    }
                    //nothing found
                    return null;
                }
                return (last + 1);
            },

            /// Function checks if extension is in any valid range
            isExtensionInRange: function (globals, val) {
                if (val == null || val === "") {
                    return false;
                }
                for (var i = 0; i < globals.validExtensionRanges.length; i++) {
                    var range = globals.validExtensionRanges[i];
                    if (val >= range.start && val <= range.end) {
                        return true;
                    }
                }
                return false;
            },

            /// Generate localized description of configured valid ranges.
            getRangesText: function (globals) {
                var lst = [],
                    len = globals.validExtensionRanges.length;
                for (var i = 0; i < len; i++) {
                    var range = globals.validExtensionRanges[i];
                    lst.push(_("from __start__ to __end__", {
                        start: range.start,
                        end: range.end
                    }));
                }
                return _("Allowed ranges: __ranges__.", { 'ranges': lst.join(", ") });
            }
        };

        return extensionTools;
    }
);