define([
    "aps/i18n",
    "aps/ResourceStore",
    "dojo/Deferred"
], function (
    _,
    Store,
    Deferred
) {
    return function (orgnizationId, apsTypes) {
        var deferredResult = new Deferred();
        var request = new Deferred();
        if (aps.context.vars.subscription) {
            request.resolve([aps.context.vars.subscription]);
        } else {
            new Store({
                target: "/aps/2/resources/" + orgnizationId + "/subscription/"
            }).query().then(function(subscriptions) {
                request.resolve(subscriptions);
            });
        }
        request.then(function(subscriptions) {
            if (subscriptions.length > 0) {
                request = new Store({
                    target: "/aps/2/resources/" + subscriptions[0].aps.id + "/resources/"
                }).query();
                request.then(function(resources) {
                    var processedResources = {};
                    if (resources.length > 0) {
                        var unlimited = "unlimited";
                        var processResource = function (resource) {
                            if (apsTypes && apsTypes.indexOf(resource.apsType) < 0) {
                                return;
                            }

                            // count available
                            if ("limit" in resource) {
                                resource.availableCount = resource.limit - resource.usage;
                                resource.isAvailable = resource.availableCount > 0;
                            } else {
                                resource.limit = unlimited;
                                resource.availableCount = unlimited;
                                resource.isAvailable = true;
                            }

                            // check whether the resource of the same apsType has been already added
                            if (resource.apsType in processedResources) {
                                var resourceGroup = processedResources[resource.apsType];
                                //process a case when a few subscription resources were created from the same instance
                                if (resource.apsId && resource.apsId in resourceGroup.apsIds) {
                                    // count the common limitation in this case
                                    var index = resourceGroup.apsIds[resource.apsId];
                                    resourceGroup[index].usage += resource.usage;
                                    if (resourceGroup[index].limit != unlimited) {
                                        if (resource.limit == unlimited) {
                                            resourceGroup[index].limit = unlimited;
                                            resourceGroup[index].availableCount = unlimited;
                                        } else {
                                            resourceGroup[index].limit += resource.limit;
                                            resourceGroup[index].availableCount += resource.availableCount;
                                        }
                                    }
                                    resourceGroup[index].isAvailable = resourceGroup.isAvailable || resource.isAvailable;
                                } else {
                                    if (resource.apsId) {
                                        processedResources[resource.apsType].apsIds[resource.apsId] = processedResources[resource.apsType].length;
                                    }

                                    processedResources[resource.apsType].push(resource);
                                }

                                // update summary availability for resources of this apsType
                                processedResources[resource.apsType].isAvailable = processedResources[resource.apsType].isAvailable || resource.isAvailable;
                                if (processedResources[resource.apsType].availableCount !== unlimited) {
                                    if (resource.availableCount === unlimited) {
                                        processedResources[resource.apsType].availableCount = unlimited;
                                    } else {
                                        processedResources[resource.apsType].availableCount += resource.availableCount;
                                    }
                                }
                            } else {
                                processedResources[resource.apsType] = [resource];
                                processedResources[resource.apsType].availableCount = resource.availableCount;
                                processedResources[resource.apsType].isAvailable = resource.isAvailable;
                                processedResources[resource.apsType].apsIds = {};
                                if (resource.apsId) {
                                    processedResources[resource.apsType].apsIds[resource.apsId] = 0;
                                }
                            }
                        };

                        resources.forEach(processResource);
                    }

                    deferredResult.resolve(processedResources);
                }, function(err) {
                    deferredResult.reject(err);
                });
            } else {
                deferredResult.reject(_("Cannot retrieve resources: organization is not binded to any subscription."));
            }
        }, function(err) {
            deferredResult.reject(err);
        });

        return deferredResult;
    };
});