define([
    "aps/i18n",
    "dojo/json",
    "dojox/html/entities",
    "aps/Output"
], function(
    _,
    json,
    entities,
    Output
) {
    "use strict";

    if (document.createStyleSheet) {
        document.createStyleSheet('styles.css');
    } else {
        var newSS = document.createElement('link');
        newSS.rel = 'stylesheet';
        newSS.href = 'styles.css';
        newSS.type = 'text/css';
        document.getElementsByTagName("head")[0].appendChild(newSS);
    }

    // defines global variable to store constant values
    if (!window.constants) {
        window.constants = {};
    }

    var utils = {
        sleep: function(ms) {
            ms += new Date().getTime();
            while (new Date() < ms) {
            }
        },
        _buildIconLabel: function(className, text) {
            return className.substring(0, 5) == "hint-" ?
                ("<div style='white-space:nowrap' class='" + className + "'><span>" + text + "</span></div>") :
                ("<div style='white-space:nowrap' class='" + className + " sbLabel'><span>" + text + "</span></div>");
        },
        _buildStatusLabel: function(className, text) {
            return utils._buildIconLabel(className, text);
        },
        getStatusLabel: function(status) {
            switch (status) {
            case "aps:ready":
                return utils._buildStatusLabel('hint-ok', _('Ready'));
            case "aps:provisioning":
                return utils._buildStatusLabel('hint-attention', _('Provisioning'));
            case "aps:deleting":
            case "aps:unprovisioning":
                return utils._buildStatusLabel('sb-disable', _('Deleting'));
            default:
                return _('Unknown status');
            }
        },
        renderGrigRowStatus: function(row, val) {
            if (!val) {
                return "";
            }
            var statusLabel = utils.getStatusLabel(val);
            return utils._createOutput(statusLabel);
        },
        _createOutput: function(str){
            return new Output({
                content: str,
                escapeHTML: false
            });
        },
        buildStatusOutput: function(className, text) {
            var statusLabel = utils._buildStatusLabel(className, text);
            return utils._createOutput(statusLabel);
        },
        renderStatus: function (statusString) {
            if (!statusString) { return ""; }
                return utils.getStatusLabel(statusString);
        },
        renderLink: function(val) {
            if (!val) {
                return "";
            }
            return utils.getLinkLabel(entities.encode(val));
        },
        _buildBooleanLabel: function(className, text) {
            return utils._buildIconLabel(className, text);
        },
        getBooleanLabel: function(flag, trueText, falseText) {
            if (flag) {
                return utils._buildBooleanLabel('sb-on', trueText);
            } else {
                return utils._buildBooleanLabel('sb-off', falseText);
            }
        },
        getBooleanOutput: function(flag, trueText, falseText) {
            var boolLabel = utils.getBooleanLabel(flag, trueText, falseText);
            return utils._createOutput(boolLabel);
        },
        getYesNoLabel: function(flag) {
            return utils.getBooleanLabel(flag, _('Yes'), _('No'));
        },
        getEnabledDisabledLabel: function(flag) {
            return utils.getBooleanLabel(flag, _('Enabled'), _('Disabled'));
        },
        getOnOffOutput: function(flag) {
            var boolLabel = utils.getBooleanLabel(flag, _('On'), _('Off'));
            return utils._createOutput(boolLabel);
        },
        getLinkLabel: function(link) {
            return "<div style='white-space:nowrap'><a target='_blank' href='" + encodeURI(link) + "'>" + link + "</a></div>";
        },
        getEmailLink: function(email) {
            return "<a target='_blank' href='mailto:" + encodeURI(email) + "'>" + email + "</a>";
        },
        getNavigationLabel: function(viewId, resourceId, label, skipEncode){
            var encondedLabel = (!skipEncode? entities.encode(label):label);
            var navigationLabel = "<a href='javascript:aps.apsc.gotoView(\"" + viewId + "\", \"" + resourceId + "\")'>" + encondedLabel + "</a>";
            return utils._createOutput(navigationLabel);
        },
        txtNoItemsFound: function() {
            return _('No item(s) found');
        },
        gotoBackView: function(fromList, listView, instanceView, id) {
            if (fromList) {
                aps.apsc.gotoView(listView);
            } else {
                aps.apsc.gotoView(instanceView, id);
            }
        },
        isEmpty: function(map) {
            for (var key in map) {
                if (map.hasOwnProperty(key)) {
                    return false;
                }
            }
            return true;
        },
        trimStr: function(str) {
            if (!String.prototype.trim) {
                String.prototype.trim = function() {
                    return this.replace(/^\s+|\s+$/gm, '');
                };
            }
            return str ? str.trim() : "";
        },
        compareStr: function(a, b) {
            if (typeof a === "undefined" || a == null) return (typeof b === "undefined" || b == null) ? 0 : -1;
            if (typeof b === "undefined" || b == null) return 1;
            if (a.toString() < b.toString()) return -1;
            if (a.toString() > b.toString()) return 1;
            return 0;
        },
        compareObj: function( x, y ) {
            if ( x === y ) return true;
            // if both x and y are null or undefined and exactly the same

            if ( ! ( x instanceof Object ) || ! ( y instanceof Object ) ) return false;
            // if they are not strictly equal, they both need to be Objects

            if ( x.constructor !== y.constructor ) return false;
            // they must have the exact same prototype chain, the closest we can do is
            // test there constructor.

            for ( var p in x ) {
                if (x.hasOwnProperty( p )) {
                    if ( ! y.hasOwnProperty( p ) ) return false;
                    // allows to compare x[ p ] and y[ p ] when set to undefined

                    if ( x[ p ] === y[ p ] ) continue;
                    // if they have the same strict value or identity then they are equal

                    if ( typeof( x[ p ] ) !== "object" ) return false;
                    // Numbers, Strings, Functions, Booleans must be strictly equal

                    if ( ! utils.compareObj( x[ p ],  y[ p ] ) ) return false;
                }
                // other properties were tested using x.constructor === y.constructor
                // Objects and Arrays must be tested recursively
            }

            for ( p in y ) {
                if ( y.hasOwnProperty( p ) && ! x.hasOwnProperty( p ) ) return false;
                // allows x[ p ] to be set to undefined
            }
            return true;
        },
        // TODO: replace regex with phone number library
        normalizePhoneNumber: function(phoneNumber, formats) {
            if (phoneNumber == null || phoneNumber === "") return "";

            //normalize string and remove all unnecessary characters
            var normalPhoneNumber = phoneNumber.replace(/[^\d]/g, "");

            for (var i = 0; i < formats.length; ++i) {
                if (normalPhoneNumber.match(formats[i].regex)) {
                    return normalPhoneNumber.replace(formats[i].regex, formats[i].pattern);
                }
            }
            return phoneNumber;
        },
        allignByHeight: function(itemA, itemB) {
            if (typeof itemA === "string") {
                itemA = document.getElementById(itemA);
            }
            if (typeof itemB === "string") {
                itemB = document.getElementById(itemB);
            }
            if (itemB.offsetHeight > itemA.offsetHeight) {
                itemA.style.minHeight = itemB.offsetHeight + 'px';
                itemB.style.minHeight = itemB.offsetHeight + 'px';
            } else {
                itemA.style.minHeight = itemA.offsetHeight + 'px';
                itemB.style.minHeight = itemA.offsetHeight + 'px';
            }
        },

        //APSA-5731 (but APSA-5993)
        emulateOptionsGroup: function(phoneNumberSelector) {
            for (var optionIndex = 0; optionIndex < phoneNumberSelector.domNode.length; optionIndex++) {
                var option = phoneNumberSelector.domNode[optionIndex];
                if (option.value === "") {
                    option.className = option.className + " hint";
                    option.innerHTML = "&nbsp;&nbsp;&nbsp;" + option.innerHTML;
                } else if (option.disabled) {
                } else {
                    option.innerHTML = "&nbsp;&nbsp;&nbsp;" + option.innerHTML;
                }
            }
        },

        // encodeURIComponent leaves -_.!~*'() characters as they are. RQL requires additional encording of !'().
        encodeComponentForRql: function(value) {
            /// <summary>
            /// Encodes text value to be accepted by RQL
            /// </summary>
            return encodeURIComponent(value).replace(/\!/g, "%21").replace(/\'/g, "%27").replace(/\(/g, "%28").replace(/\)/g, "%29");
        },
        trimOnChange: function(newValue) {
            /// <summary>
            /// Trims value onChange event of TextBox widget.
            /// Note: cannot be used for TextArea
            /// </summary>
            this.set('value', utils.trimStr(newValue));
        },
        getErrorMessage: function(err) {
            if (typeof err == 'string' || err instanceof String) {
                return err;
            }
            var errData = {};
            try {
                errData = json.parse(err.response.text);
            } catch(e) {
            }

            return errData.message ?
                (!("status" in err) || err.status === null ? "" : "[" + err.status + "] ") + errData.message.replace("REST Application returns error code=500: ", "") :
                err.toString();
        },
        showLoading: function() {
            if (aps.apsc.showLoading) {
                aps.apsc.showLoading();
            }
        },
        hideLoading: function() {
            if (aps.apsc.hideLoading) {
                aps.apsc.hideLoading();
            }
        },
        validateSelectedAudioFile: function(files) {
            if (!files || files.length < 1) {
                return false;
            }

            if (files[0].size > 4 * 1024 * 1024) {
                alert(_("The size of the selected file is more then 4MB. Please, select other file."));
                return false;
            }

            var supportedFormats = utils.getAudioSupportedFormatsString();
            if (supportedFormats.indexOf(files[0].type) == -1) {
                var message = _("The selected file {fileName} is not a WAV or WMA file. Please, select other file.");
                message = message.replace("{fileName}", files[0].name);
                alert(message);
                return false;
            }

            return true;
        },
        isFileApiSupported: function() {
            return !!(window.FileReader && window.File && window.FileList && window.Blob);
        },
        isInternetExplorer: function() {
            var userAgent = window.navigator.userAgent;
            return userAgent.indexOf('MSIE') > 0 || userAgent.match(/Trident.*rv\:11\./);
        },
        isMacintoshCrome: function() {
            var userAgent = window.navigator.userAgent;
            return userAgent.indexOf('Macintosh') > 0 && userAgent.indexOf('Intel Mac OS X') > 0 && userAgent.indexOf('Chrome') > 0;
        },
        getAudioSupportedFormatsString: function() {
            var formats = "audio/wav, audio/x-ms-wma, audio/x-wav";
            if (utils.isMacintoshCrome()) {
                formats += ", video/x-ms-wma"; // APSA-6385
            }
            return formats;
        },
        getFileName: function(fullPath) {
            return fullPath.replace(/^.*[\\\/]/, '');
        },
        parseFileReaderResult: function(evnResult) {
            // summary:
            //      parses onloadend event result of FileReader and returns base64 encoded content.
            //      expects that content is read by readAsBinaryString or readAsDataURL methods.

            if (evnResult.indexOf("data:") === 0) {
                //  regexp works too slow with a big data. have to use substring.
                //  readAsDataURL returns base64 content, so ;base64, tag always here
                var base64Item = ';base64,';
                var i = evnResult.indexOf(base64Item);
                return evnResult.substring(i + base64Item.length);
            } else {
                return window.btoa(evnResult);
            }
        },
        getApsInstanceStatusFilter: function() {
            return {
                "title": _("Status"),
                "options": [
                    { "value": "aps:ready", "label": _("Ready") },
                    { "value": "aps:provisioning", "label": _("Provisioning") },
                    { "value": 3, "label": _("Configuring") },
                    { "value": "aps:unprovisioning", "label": _("Deleting") }
                ]
            };
        },
        getFeatureIcons: function(lyncUserProfileId, userProfile) {
            // summary:
            //      get the feature icons of license basing on lyncUserProfileId.
            if (lyncUserProfileId) {
                var featuresStr = "&nbsp;<i class='icon'><img src='images/16/online-chat.png' title='" + utils.escapeHTML(_("Instant Messaging & Presence")) + "'></i>";

                var featureDisabled;
                var featureTitle;

                if (userProfile._voiceCalls) {
                    featureDisabled = userProfile._voiceCalls == "zdisabled" || userProfile._voiceCalls == "znotIncluded";
                    featureTitle = userProfile._voiceCalls == "znotIncluded" ? _("Phone Calls (Not Included in License)") : _("Phone Calls");
                    var voiceIcon = "&nbsp;<i class='icon'><img src='images/16/" + (featureDisabled ? "phone-calls-disabled.png" : "phone-calls.png") + "' title='" + featureTitle + "'></i>";
                    featuresStr += "<span class='hint noblock'>" + voiceIcon + "</span>";
                }

                featureDisabled = userProfile._onlineMeetings !== "Enabled";
                featureTitle = featureDisabled ? "Online Meetings (Not Included in License)" : _("Online Meetings");
                var meetingIcon = "&nbsp;<i class='icon'><img src='images/16/" + (featureDisabled ? "online-meeting-disabled.png" : "online-meeting.png") + "' title='" + featureTitle + "'></i>";
                featuresStr += "<span class='hint noblock'>" + meetingIcon + "</span>";

                return _("Features: __features__", { features: featuresStr });
            } else {
                return "";
            }
        },

        escapeHTML: function(text){
            var output = new Output({value:text});
            return output.value;
        }
    };

    return utils;
});

